@@ -40,6 +40,9 @@
 
 #include "unit_data_block.h"
 #include <algorithm>
+#include <forward_list>
+#include <list>
+#include <numeric>
 
 /**
  * @todo Use proper checks for preprocessor directives to selectively include
@@ -56,11 +59,49 @@
 #include "experimental/tiledb/common/dag/utility/range_join.h"
 using namespace tiledb::common;
 
+TEST_CASE(
+    "DataBlock: Test create DataBlock with constructors", "[data_block]") {
+  [[maybe_unused]] auto da = DataBlock<4'096>{};
+  CHECK(da.size() == 0);
+  CHECK(da.capacity() == 4096);
+}
+
+TEST_CASE("DataBlock: Test create DataBlock new / delete", "[data_block]") {
+  auto da = new DataBlock<4'096>;
+  CHECK(da->size() == 0);
+  CHECK(da->capacity() == 4096);
+  delete da;
+}
+
+TEST_CASE("DataBlock: Test create DataBlock with make_unique", "[data_block]") {
+  auto da = std::make_unique<DataBlock<4'096>>();
+  CHECK(da->size() == 0);
+  CHECK(da->capacity() == 4096);
+}
+
+TEST_CASE("DataBlock: Test create DataBlock with make_shared", "[data_block]") {
+  auto da = std::make_shared<DataBlock<4'096>>();
+  CHECK(da->size() == 0);
+  CHECK(da->capacity() == 4096);
+}
+
+TEST_CASE("DataBlock: Test with forward_list", "[data_block]") {
+  std::forward_list<DataBlock<4'096>> slist;
+  slist.emplace_front();
+  CHECK(slist.front().size() == 0);
+  CHECK(slist.front().capacity() == 4096);
+
+  // This is okay since `DataBlock`s are shared ptrs to underlying memory.
+  // We don't need an array new.
+  std::vector<DataBlock<4'096>> vec{4};
+}
+
 /**
  * Test constructor
  */
 TEST_CASE("DataBlock: Test create DataBlock", "[data_block]") {
   SECTION("Constructor compilation tests") {
+    using DataBlock = DataBlock<4'096>;
     DataBlock da;
     CHECK(da.size() == 0);
     CHECK(da.capacity() == DataBlock::max_size());
@@ -168,6 +209,9 @@ void db_test_1(const DB& db) {
  * Invoke simple API tests with a basic `DataBlock`
  */
 TEST_CASE("DataBlock: Test API of variously sized DataBlock", "[data_block]") {
+  using DataBlock = DataBlock<4'096>;
+  size_t chunk_size_ = 4'096;
+
   SECTION("Specific size constructed") {
     auto db = DataBlock{1};
     db_test_0(db);
@@ -205,27 +249,6 @@ TEST_CASE("DataBlock: Test API of variously sized DataBlock", "[data_block]") {
   }
 }
 
-/**
- * Invoke the simple tests with a `DataBlock` created with an `std::allocator`.
- */
-TEST_CASE(
-    "DataBlock: Test create DataBlock with std::allocator<std::byte>",
-    "[data_block]") {
-  auto db = DataBlockImpl<std::allocator<std::byte>>{chunk_size_};
-  CHECK(db.size() == chunk_size_);
-  CHECK(db.capacity() == DataBlock::max_size());
-
-  db_test_0(db);
-  db_test_1(db);
-
-  auto dc = DataBlock{chunk_size_};
-  CHECK(dc.size() == chunk_size_);
-  CHECK(dc.capacity() == DataBlock::max_size());
-
-  db_test_0(dc);
-  db_test_1(dc);
-}
-
 /**
  * Test iterating through a `DataBlock`
  */
@@ -251,7 +274,9 @@ void db_test_2(DB& db) {
  * `PoolAllocator`.
  */
 TEST_CASE("DataBlock: Iterate through data_block", "[data_block]") {
-  auto db = DataBlockImpl<std::allocator<std::byte>>{};
+  static constexpr size_t chunk_size_ = 4'096;
+  using DataBlock = DataBlock<chunk_size_>;
+  auto db = DataBlock{};
   db_test_2(db);
   auto dc = DataBlock{chunk_size_};
   db_test_2(dc);
@@ -262,12 +287,13 @@ TEST_CASE("DataBlock: Iterate through data_block", "[data_block]") {
  * `std::allocator` and with our `PoolAllocator`.
  */
 TEST_CASE("DataBlock: Iterate through 8 data_blocks", "[data_block]") {
+  using DataBlock = DataBlock<4'096>;
   for (size_t i = 0; i < 8; ++i) {
-    auto db = DataBlockImpl<std::allocator<std::byte>>{};
+    auto db = DataBlock{};
     db_test_2(db);
   }
   for (size_t i = 0; i < 8; ++i) {
-    auto db = DataBlock{chunk_size_};
+    auto db = DataBlock{4'096};
     db_test_2(db);
   }
 }
@@ -276,6 +302,8 @@ TEST_CASE("DataBlock: Iterate through 8 data_blocks", "[data_block]") {
  * Verify some properties of `DataBlock`s
  */
 TEST_CASE("DataBlock: Get span", "[data_block]") {
+  using DataBlock = DataBlock<4'194'304>;
+
   auto a = DataBlock{};
   auto b = DataBlock{};
   auto c = DataBlock{};
@@ -324,6 +352,9 @@ TEST_CASE("DataBlock: Get span", "[data_block]") {
  * Test resizing.
  */
 TEST_CASE("DataBlock: Test resize", "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<4'194'304>;
+
   auto a = DataBlock{chunk_size_};
   auto b = DataBlock{chunk_size_};
   auto c = DataBlock{chunk_size_};
@@ -346,6 +377,8 @@ TEST_CASE("DataBlock: Test resize", "[data_block]") {
 }
 
 TEST_CASE("DataBlock: Test (shallow) copying", "[data_block]") {
+  using DataBlock = DataBlock<4'194'304>;
+
   auto a = DataBlock{DataBlock::max_size()};
   auto b = DataBlock{DataBlock::max_size()};
   auto c = DataBlock{DataBlock::max_size()};
@@ -519,6 +552,8 @@ TEST_CASE(
     "[data_block]") {
   std::byte* x;
   std::byte* y;
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
 
   {
     auto a = DataBlock{chunk_size_};
@@ -545,6 +580,9 @@ TEST_CASE(
 TEST_CASE(
     "DataBlock: Test DataBlock allocation/deallocation from pool, etc",
     "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{chunk_size_};
   auto b = DataBlock{chunk_size_};
   auto c = DataBlock{chunk_size_};
@@ -584,6 +622,9 @@ TEST_CASE(
 TEST_CASE(
     "DataBlock: Test DataBlock allocation/deallocation on copying, etc",
     "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{chunk_size_};
   auto b = DataBlock{chunk_size_};
   auto c = DataBlock{chunk_size_};
@@ -677,6 +718,9 @@ TEST_CASE(
  * `std::fill` algorithm.
  */
 void test_std_fill(size_t test_size) {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{test_size};
   auto b = DataBlock{test_size};
   auto c = DataBlock{test_size};
@@ -749,6 +793,8 @@ void test_std_fill(size_t test_size) {
  * `std::fill` algorithm.
  */
 TEST_CASE("DataBlock: Fill with std::fill", "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+
   SECTION("chunk_size") {
     test_std_fill(chunk_size_);
   }
@@ -764,6 +810,9 @@ TEST_CASE("DataBlock: Fill with std::fill", "[data_block]") {
  * Verify some properties of joined `DataBlock`s.
  */
 void test_join(size_t test_size) {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{test_size};
   auto b = DataBlock{test_size};
   auto c = DataBlock{test_size};
@@ -824,6 +873,8 @@ void test_join(size_t test_size) {
 }
 
 TEST_CASE("DataBlock: Join data_blocks (join view)", "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+
   SECTION("chunk_size") {
     test_join(chunk_size_);
   }
@@ -840,6 +891,9 @@ TEST_CASE("DataBlock: Join data_blocks (join view)", "[data_block]") {
  * library algorithms.
  */
 void test_join_std_fill(size_t test_size) {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{test_size};
   auto b = DataBlock{test_size};
   auto c = DataBlock{test_size};
@@ -876,6 +930,8 @@ void test_join_std_fill(size_t test_size) {
 }
 
 TEST_CASE("DataBlock: Join data_blocks std::fill", "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+
   SECTION("chunk_size") {
     test_join_std_fill(chunk_size_);
   }
@@ -891,6 +947,9 @@ TEST_CASE("DataBlock: Join data_blocks std::fill", "[data_block]") {
  * Test `operator[]` of joined `DataBlock`s.
  */
 void test_join_operator_bracket(size_t test_size) {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{test_size};
   auto b = DataBlock{test_size};
   auto c = DataBlock{test_size};
@@ -933,6 +992,8 @@ void test_join_operator_bracket(size_t test_size) {
 }
 
 TEST_CASE("DataBlock: Join data_blocks operator[]", "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+
   SECTION("chunk_size") {
     test_join_operator_bracket(chunk_size_);
   }
@@ -948,6 +1009,9 @@ TEST_CASE("DataBlock: Join data_blocks operator[]", "[data_block]") {
  * Additional standard library uses of joined `DataBlock`s.
  */
 void test_operator_bracket_loops(size_t test_size) {
+  constexpr size_t chunk_size_ = 4'194'304;
+  using DataBlock = DataBlock<chunk_size_>;
+
   auto a = DataBlock{test_size};
   auto b = DataBlock{test_size};
   auto c = DataBlock{test_size};
@@ -1024,6 +1088,8 @@ void test_operator_bracket_loops(size_t test_size) {
 }
 
 TEST_CASE("DataBlock: Join data_blocks loops operator[]", "[data_block]") {
+  constexpr size_t chunk_size_ = 4'194'304;
+
   SECTION("chunk_size") {
     test_operator_bracket_loops(chunk_size_);
   }
