@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2022 TileDB Inc.
+ * @copyright Copyright (c) 2022-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,7 +39,6 @@
 #include <test/support/src/vfs_helpers.h>
 #include <test/support/tdb_catch.h>
 #include "tiledb/sm/cpp_api/tiledb"
-#include "tiledb/sm/misc/utils.h"
 
 using namespace tiledb;
 
@@ -224,10 +223,12 @@ void create_array(
       .set_data_buffer("a", a_data_read)
       .set_data_buffer("b", b_data_read);
 
+  Subarray subarray1(ctx, array1);
   if (array_type == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query1.add_range("rows", range[0], range[1])
+    subarray1.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query1.set_subarray(subarray1);
   }
   query1.submit();
 
@@ -366,10 +367,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -430,10 +433,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -559,10 +564,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -686,8 +693,10 @@ TEST_CASE(
 
   // Define range.
   int range[] = {2, 3};
-  query.add_range("rows", range[0], range[1])
+  Subarray subarray(ctx2, array);
+  subarray.add_range("rows", range[0], range[1])
       .add_range("cols", range[0], range[1]);
+  query.set_subarray(subarray);
 
   // Perform query and validate.
   perform_query(a_data_read_2, b_data_read_2, qc, params.layout_, query);
@@ -816,8 +825,10 @@ TEST_CASE(
   // Define range.
   int range[] = {2, 3};
   int range1[] = {7, 10};
-  query.add_range("rows", range1[0], range1[1])
+  Subarray subarray(ctx2, array);
+  subarray.add_range("rows", range1[0], range1[1])
       .add_range("cols", range[0], range[1]);
+  query.set_subarray(subarray);
 
   // Perform query and validate.
   perform_query(a_data_read_2, b_data_read_2, qc, params.layout_, query);
@@ -955,8 +966,10 @@ TEST_CASE(
   // Define range.
   int range[] = {2, 3};
   int range1[] = {7, 10};
-  query.add_range("rows", range[0], range[1])
+  Subarray subarray(ctx2, array);
+  subarray.add_range("rows", range[0], range[1])
       .add_range("cols", range1[0], range1[1]);
+  query.set_subarray(subarray);
 
   // Perform query and validate.
   perform_query(a_data_read_2, b_data_read_2, qc, params.layout_, query);
@@ -1079,8 +1092,10 @@ TEST_CASE(
 
   // Define range.
   int range[] = {7, 14};
-  query.add_range("rows", range[0], range[1])
+  Subarray subarray(ctx2, array);
+  subarray.add_range("rows", range[0], range[1])
       .add_range("cols", range[0], range[1]);
+  query.set_subarray(subarray);
 
   // Perform query and validate.
   perform_query(a_data_read_2, b_data_read_2, qc, params.layout_, query);
@@ -1241,8 +1256,10 @@ TEST_CASE(
 
   // Define range.
   int range[] = {7, 14};
-  query.add_range("rows", range[0], range[1])
+  Subarray subarray(ctx2, array);
+  subarray.add_range("rows", range[0], range[1])
       .add_range("cols", range[0], range[1]);
+  query.set_subarray(subarray);
 
   // Perform query and validate.
   perform_query(a_data_read_2, b_data_read_2, qc, params.layout_, query);
@@ -1413,8 +1430,10 @@ TEST_CASE(
 
   // Define range.
   int range[] = {7, 14};
-  query.add_range("rows", range[0], range[1])
+  Subarray subarray(ctx2, array);
+  subarray.add_range("rows", range[0], range[1])
       .add_range("cols", range[0], range[1]);
+  query.set_subarray(subarray);
 
   // Perform query and validate.
   perform_query(
@@ -1542,18 +1561,22 @@ TEST_CASE(
   std::vector<int> vals = {3, 3, 3, 3, 3, 3, 3, 3, 3, 3};
   Query query_w(ctx, array, TILEDB_WRITE);
   int range[] = {1, 10};
-  query_w.add_range("d", range[0], range[1]);
+  Subarray subarray_w(ctx, array);
+  subarray_w.add_range("d", range[0], range[1]);
+  query_w.set_subarray(subarray_w);
   query_w.set_layout(TILEDB_ROW_MAJOR);
-  query_w.set_buffer("a", vals);
+  query_w.set_data_buffer("a", vals);
   REQUIRE(query_w.submit() == Query::Status::COMPLETE);
 
   // Write values from 3-6 as 7 in the array.
   std::vector<int> vals2 = {7, 7, 7, 7};
   Query query_w2(ctx, array, TILEDB_WRITE);
   int range2[] = {3, 6};
-  query_w2.add_range("d", range2[0], range2[1]);
+  Subarray subarray_w2(ctx, array);
+  subarray_w2.add_range("d", range2[0], range2[1]);
+  query_w2.set_subarray(subarray_w2);
   query_w2.set_layout(TILEDB_ROW_MAJOR);
-  query_w2.set_buffer("a", vals2);
+  query_w2.set_data_buffer("a", vals2);
   REQUIRE(query_w2.submit() == Query::Status::COMPLETE);
 
   array.close();
@@ -1568,9 +1591,11 @@ TEST_CASE(
 
   std::vector<int> vals_read(10);
   Query query_r(ctx, array_r, TILEDB_READ);
-  query_r.add_range("d", range[0], range[1]);
+  Subarray subarray_r(ctx, array_r);
+  subarray_r.add_range("d", range[0], range[1]);
+  query_r.set_subarray(subarray_r);
   query_r.set_layout(TILEDB_ROW_MAJOR);
-  query_r.set_buffer("a", vals_read);
+  query_r.set_data_buffer("a", vals_read);
   query_r.set_condition(qc);
   REQUIRE(query_r.submit() == Query::Status::COMPLETE);
 
@@ -1584,6 +1609,82 @@ TEST_CASE(
   }
 }
 
+TEST_CASE(
+    "Testing sparse query condition with the same fragment domain.",
+    "[query][query-condition][sparse][same-fd]") {
+  Context ctx;
+  VFS vfs(ctx);
+
+  if (vfs.is_dir(array_name)) {
+    vfs.remove_dir(array_name);
+  }
+
+  // Create a simple 1D vector with domain 1-10 and one attribute.
+  Domain domain(ctx);
+  domain.add_dimension(Dimension::create<int>(ctx, "d", {{1, 10}}, 5));
+  ArraySchema schema(ctx, TILEDB_SPARSE);
+  schema.set_domain(domain);
+  schema.set_cell_order(TILEDB_HILBERT);
+  schema.set_allows_dups(false);
+  schema.set_capacity(2);
+
+  Attribute attr = Attribute::create<int>(ctx, "a");
+  Attribute attr2 = Attribute::create<int>(ctx, "a2");
+  int fill_value = -1;
+  attr.set_fill_value(&fill_value, sizeof(int));
+  schema.add_attribute(attr);
+  schema.add_attribute(attr2);
+  Array::create(array_name, schema);
+
+  // Open array for write.
+  Array array(ctx, array_name, TILEDB_WRITE);
+
+  std::vector<int> dim_vals = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
+
+  // Write all values as 3 in the array.
+  std::vector<int> vals = {3, 3, 3, 3, 3, 3, 3, 3, 3, 3};
+  Query query_w(ctx, array, TILEDB_WRITE);
+  query_w.set_data_buffer("d", dim_vals);
+  query_w.set_data_buffer("a", vals);
+  query_w.set_data_buffer("a2", vals);
+  REQUIRE(query_w.submit() == Query::Status::COMPLETE);
+
+  // Write values from 1-10 as 7 in the array.
+  std::vector<int> vals2 = {7, 7, 7, 7, 7, 7, 7, 7, 7, 6};
+  Query query_w2(ctx, array, TILEDB_WRITE);
+  query_w2.set_data_buffer("d", dim_vals);
+  query_w2.set_data_buffer("a", vals2);
+  query_w2.set_data_buffer("a2", vals);
+  REQUIRE(query_w2.submit() == Query::Status::COMPLETE);
+
+  array.close();
+
+  // Open the array for read.
+  Array array_r(ctx, array_name, TILEDB_READ);
+
+  // Query the data with query condition a < 7.
+  QueryCondition qc(ctx);
+  int val1 = 7;
+  qc.init("a", &val1, sizeof(int), TILEDB_LT);
+
+  std::vector<int> vals_read(10);
+  std::vector<int> vals_read2(10);
+  std::vector<int> dim_vals_read(10);
+  Query query_r(ctx, array_r, TILEDB_READ);
+  query_r.set_layout(TILEDB_GLOBAL_ORDER);
+  query_r.set_data_buffer("a", vals_read);
+  query_r.set_data_buffer("a2", vals_read2);
+  query_r.set_data_buffer("d", dim_vals_read);
+  query_r.set_condition(qc);
+  REQUIRE(query_r.submit() == Query::Status::COMPLETE);
+
+  array_r.close();
+
+  if (vfs.is_dir(array_name)) {
+    vfs.remove_dir(array_name);
+  }
+}
+
 TEST_CASE(
     "Testing read query with basic QC, condition on dimension, with range "
     "within a tile.",
@@ -1657,10 +1758,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -1801,10 +1904,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -1959,10 +2064,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -2101,10 +2208,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -2281,10 +2390,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -2505,10 +2616,12 @@ TEST_CASE(
   Query query(ctx2, array);
 
   // Set a subarray for dense.
+  Subarray subarray(ctx2, array);
   if (params.array_type_ == TILEDB_DENSE) {
     int range[] = {1, num_rows};
-    query.add_range("rows", range[0], range[1])
+    subarray.add_range("rows", range[0], range[1])
         .add_range("cols", range[0], range[1]);
+    query.set_subarray(subarray);
   }
 
   // Perform query and validate.
@@ -2611,7 +2724,9 @@ TEST_CASE(
   std::vector<uint8_t> rlabs(10);
 
   Query rquery(ctx, rarray, TILEDB_READ);
-  rquery.set_subarray(subarray)
+  Subarray sub(ctx, rarray);
+  sub.set_subarray(subarray);
+  rquery.set_subarray(sub)
       .set_layout(TILEDB_ROW_MAJOR)
       .set_data_buffer("rows", rrows)
       .set_data_buffer("labs", rlabs);
