@@ -44,6 +44,9 @@
 #else
 #include "tiledb/sm/filesystem/posix.h"
 #endif
+#include "tiledb/api/c_api/array/array_api_internal.h"
+#include "tiledb/api/c_api/array_schema/array_schema_api_external.h"
+#include "tiledb/api/c_api/array_schema/array_schema_api_internal.h"
 #include "tiledb/api/c_api/buffer/buffer_api_internal.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/common/stdx_string.h"
@@ -978,12 +981,19 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_WRITE);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  tiledb_subarray_t* sub;
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_upd, &buffer_upd_size);
@@ -996,6 +1006,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
 
   std::vector<uint64_t> fragment_timestamps;
   rc = tiledb_vfs_ls(
@@ -1035,13 +1046,19 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
-  // Submit query
+  // Create subarray
   int64_t subarray_read[] = {1, 10};
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
+  // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1054,6 +1071,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
 
   // Check correctness
   int32_t buffer_read_c[] = {1, 2, 3, 4, 50, 60, 70, 8, 9, 10};
@@ -1094,12 +1112,18 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   CHECK(timestamp_get == 0);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1112,6 +1136,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1145,12 +1170,18 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1163,6 +1194,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1202,12 +1234,18 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   CHECK(timestamp_get == fragment_timestamps[1]);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1217,6 +1255,7 @@ TEST_CASE_METHOD(
 
   // Clean up but don't close the array yet (we will reopen it).
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1233,12 +1272,18 @@ TEST_CASE_METHOD(
   rc = tiledb_array_reopen(ctx_, array);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1248,6 +1293,7 @@ TEST_CASE_METHOD(
 
   // Clean up but don't close the array yet (we will reopen it).
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
 
   // Check correctness
   int buffer_read_reopen_c[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};
@@ -1267,12 +1313,18 @@ TEST_CASE_METHOD(
   rc = tiledb_array_reopen(ctx_, array);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1285,6 +1337,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_query_free(&query);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&sub);
 
   // Check correctness
   int buffer_read_reopen_start_c[] = {
@@ -1331,12 +1384,18 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1349,6 +1408,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_query_free(&query);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1395,12 +1455,18 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1413,6 +1479,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_query_free(&query);
   tiledb_array_free(&array);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1558,13 +1625,20 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   CHECK(timestamp_get == 0);
 
+  // Create subarray
+  int64_t subarray_read[] = {1, 10};
+  tiledb_subarray_t* sub;
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  int64_t subarray_read[] = {1, 10};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   int buffer_read[10];
   uint64_t buffer_read_size = sizeof(buffer_read);
@@ -1579,6 +1653,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1612,12 +1687,18 @@ TEST_CASE_METHOD(
   rc = tiledb_array_open(ctx_, array, TILEDB_READ);
   CHECK(rc == TILEDB_OK);
 
+  // Create subarray
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray_read);
+  CHECK(rc == TILEDB_OK);
+
   // Submit query
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_ROW_MAJOR);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray_read);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
   CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
       ctx_, query, "a", buffer_read, &buffer_read_size);
@@ -1630,6 +1711,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&sub);
   tiledb_config_free(&cfg);
 
   // Check correctness
@@ -1917,9 +1999,9 @@ TEST_CASE_METHOD(
 
 TEST_CASE_METHOD(
     ArrayFx,
-    "C API: Test query errors, getting subarray info from write queries in "
+    "C API: Test subarray errors, getting subarray info from write queries in "
     "sparse arrays",
-    "[capi][query][error][sparse][rest]") {
+    "[capi][subarray][error][sparse][rest]") {
   std::string temp_dir = fs_vec_[0]->temp_dir();
   std::string array_name =
       vfs_array_uri(fs_vec_[0], temp_dir + "query_error_sparse");
@@ -1937,17 +2019,20 @@ TEST_CASE_METHOD(
   // Prepare query
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
+
+  // Prepare subarray
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
   CHECK(rc == TILEDB_OK);
   uint64_t range_num;
 
-  // Note: deprecated function.
-  rc = tiledb_query_get_range_num(ctx_, query, 0, &range_num);
+  rc = tiledb_subarray_get_range_num(ctx_, subarray, 0, &range_num);
   CHECK(rc == TILEDB_OK);
   CHECK(range_num == 1);
 
   // Note: deprecated function.
   const void *start, *end, *stride;
-  rc = tiledb_query_get_range(ctx_, query, 0, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 0, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*static_cast<const int64_t*>(start) == -1);
   CHECK(*static_cast<const int64_t*>(end) == 2);
@@ -1955,10 +2040,12 @@ TEST_CASE_METHOD(
 
   int64_t s = 10;
   int64_t e = 20;
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_ERR);
-  int64_t subarray[] = {-1, 2};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  int64_t sub[] = {-1, 2};
+  rc = tiledb_subarray_set_subarray(ctx_, subarray, sub);
+  CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
   CHECK(rc == TILEDB_ERR);
 
   // Close array
@@ -1968,6 +2055,7 @@ TEST_CASE_METHOD(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 
   remove_temp_dir(temp_dir);
 }
@@ -1993,45 +2081,50 @@ TEST_CASE_METHOD(
   int32_t a[] = {1, 2, 3, 4};
   uint64_t a_size = sizeof(a);
 
-  // Prepare query
+  // Prepare query/subarray
   tiledb_query_t* query;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_WRITE, &query);
   CHECK(rc == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
   CHECK(rc == TILEDB_OK);
   uint64_t range_num;
-  rc = tiledb_query_get_range_num(ctx_, query, 0, &range_num);
+  rc = tiledb_subarray_get_range_num(ctx_, subarray, 0, &range_num);
   CHECK(rc == TILEDB_OK);
   CHECK(range_num == 1);  // The default
   const void *start, *end, *stride;
-  rc = tiledb_query_get_range(ctx_, query, 0, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 0, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*(const uint64_t*)start == 1);
   CHECK(*(const uint64_t*)end == 10);
   int64_t s = 1;
   int64_t e = 2;
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_OK);
 
-  int64_t subarray[] = {2, 3, 4, 5};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  int64_t sub[] = {2, 3, 4, 5};
+  rc = tiledb_subarray_set_subarray(ctx_, subarray, sub);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_OK);
-  rc = tiledb_query_get_range_num(ctx_, query, 0, &range_num);
+  rc = tiledb_subarray_get_range_num(ctx_, subarray, 0, &range_num);
   CHECK(rc == TILEDB_OK);
   CHECK(range_num == 2);
-  rc = tiledb_query_get_range(ctx_, query, 0, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 0, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*(const uint64_t*)start == 2);
   CHECK(*(const uint64_t*)end == 3);
-  rc = tiledb_query_get_range(ctx_, query, 1, 0, &start, &end, &stride);
+  rc = tiledb_subarray_get_range(ctx_, subarray, 1, 0, &start, &end, &stride);
   CHECK(rc == TILEDB_OK);
   CHECK(*(const uint64_t*)start == 4);
   CHECK(*(const uint64_t*)end == 5);
 
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_ERR);
 
@@ -2042,6 +2135,7 @@ TEST_CASE_METHOD(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 
   remove_temp_dir(temp_dir);
 }
@@ -2113,16 +2207,21 @@ TEST_CASE_METHOD(
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
   CHECK(rc == TILEDB_OK);
 
-  int64_t subarray[] = {2, 3, 4, 5};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  tiledb_subarray_t* subarray;
+  rc = tiledb_subarray_alloc(ctx_, array, &subarray);
+  CHECK(rc == TILEDB_OK);
+  int64_t sub[] = {2, 3, 4, 5};
+  rc = tiledb_subarray_set_subarray(ctx_, subarray, sub);
   CHECK(rc == TILEDB_OK);
   int64_t s = 1;
   int64_t e = 2;
-  rc = tiledb_query_add_range(ctx_, query, 0, &s, &e, nullptr);
+  rc = tiledb_subarray_add_range(ctx_, subarray, 0, &s, &e, nullptr);
   CHECK(rc == TILEDB_OK);
 
   rc = tiledb_query_set_layout(ctx_, query, TILEDB_GLOBAL_ORDER);
   CHECK(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, subarray);
+  CHECK(rc == TILEDB_OK);
   rc = tiledb_query_submit(ctx_, query);
   CHECK(rc == TILEDB_ERR);
 
@@ -2133,6 +2232,7 @@ TEST_CASE_METHOD(
   // Clean up
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 
   remove_temp_dir(temp_dir);
 }
@@ -2197,7 +2297,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
 
   // Get a reference value to check against after deserialization
-  auto all_arrays = array->array_->array_schemas_all();
+  auto all_arrays = array->array_schemas_all();
 
   // Close array
   rc = tiledb_array_close(ctx_, array);
@@ -2211,9 +2311,9 @@ TEST_CASE_METHOD(
   // in array open v1 but with separate requests, so we simulate
   // this here by forcing metadata loading
   if (!array_v2) {
-    auto metadata = &array->array_->metadata();
+    auto metadata = &array->metadata();
     CHECK(metadata != nullptr);
-    array->array_->non_empty_domain();
+    array->non_empty_domain();
   }
 
   // Serialize array and deserialize into new_array
@@ -2231,7 +2331,7 @@ TEST_CASE_METHOD(
   tiledb_query_free(&query);
 
   // Check the retrieved array schema
-  auto& new_array_schema = new_array->array_->array_schema_latest();
+  auto& new_array_schema = new_array->array_schema_latest();
 
   auto cell_order = new_array_schema.cell_order();
   CHECK(cell_order == Layout::ROW_MAJOR);
@@ -2246,7 +2346,7 @@ TEST_CASE_METHOD(
   CHECK(ndim == 1);
 
   // Check all the retrieved arrays
-  auto all_arrays_new = new_array->array_->array_schemas_all();
+  auto all_arrays_new = new_array->array_schemas_all();
   CHECK(all_arrays.size() == all_arrays_new.size());
   CHECK(std::equal(
       all_arrays.begin(),
@@ -2255,14 +2355,14 @@ TEST_CASE_METHOD(
       [](auto a, auto b) { return a.first == b.first; }));
 
   // Check the retrieved non empty domain
-  auto non_empty_domain = new_array->array_->loaded_non_empty_domain();
+  auto non_empty_domain = new_array->loaded_non_empty_domain();
   CHECK(non_empty_domain.empty() == false);
 
   // Check the retrieved metadata
   Datatype type;
   const void* v_r;
   uint32_t v_num;
-  auto new_metadata = &new_array->array_->metadata();
+  auto new_metadata = &new_array->metadata();
   new_metadata->get("aaa", &type, &v_num, &v_r);
   CHECK(static_cast<tiledb_datatype_t>(type) == TILEDB_INT32);
   CHECK(v_num == 1);
@@ -2472,7 +2572,7 @@ TEST_CASE_METHOD(
   // Simulate serializing array_open request to Server and deserializing on
   // server side. First set the query_type that will be serialized
   auto query_type_w = QueryType::WRITE;
-  array->array_->set_query_type(query_type_w);
+  array->set_query_type(query_type_w);
   tiledb_array_t* deserialized_array_server = nullptr;
   // 1. Client -> Server : Send array_open_request (serialize)
   // 2. Server : Receive and deserialize array_open_request
@@ -2484,14 +2584,14 @@ TEST_CASE_METHOD(
   REQUIRE(rc == TILEDB_OK);
 
   // Check that the original and de-serialized array have the same query type
-  REQUIRE(deserialized_array_server->array_->get_query_type() == query_type_w);
+  REQUIRE(deserialized_array_server->get_query_type() == query_type_w);
 
   // 3. Server: Open the array the request was received for in the requested
   // mode
   // This is needed in test, as the deserialized array has a dummy array_uri of
   // "deserialized_array" set instead of the original one. The Cloud side
   // already knows the array URI so it's not a problem in the real life scenario
-  deserialized_array_server->array_->set_array_uri(array->array_->array_uri());
+  deserialized_array_server->set_array_uri(array->array_uri());
   rc = tiledb_array_open(
       ctx_,
       deserialized_array_server,
@@ -2508,13 +2608,12 @@ TEST_CASE_METHOD(
       1,
       &buff);
   REQUIRE(rc == TILEDB_OK);
-  auto st = tiledb::sm::serialization::array_deserialize(
-      array->array_.get(),
+  tiledb::sm::serialization::array_deserialize(
+      array->array().get(),
       tiledb::sm::SerializationType::CAPNP,
       buff->buffer(),
-      ctx_->storage_manager(),
+      ctx_->context().resources(),
       memory_tracker_);
-  REQUIRE(st.ok());
 
   // 6. Server: Close array and clean up
   rc = tiledb_array_close(ctx_, deserialized_array_server);
@@ -2617,11 +2716,11 @@ TEST_CASE_METHOD(
   SECTION("delete_fragments") {
     // Serialize fragment timestamps and deserialize delete request
     tiledb::sm::serialization::serialize_delete_fragments_timestamps_request(
-        array->array_->config(),
+        array->config(),
         start_timestamp,
         end_timestamp,
         tiledb::sm::SerializationType::CAPNP,
-        &buff->buffer());
+        buff->buffer());
     rc = tiledb_handle_array_delete_fragments_timestamps_request(
         ctx_,
         array,
@@ -2654,10 +2753,10 @@ TEST_CASE_METHOD(
 
     // Serialize fragments list and deserialize delete request
     tiledb::sm::serialization::serialize_delete_fragments_list_request(
-        array->array_->config(),
+        array->config(),
         fragments,
         tiledb::sm::SerializationType::CAPNP,
-        &buff->buffer());
+        buff->buffer());
     rc = tiledb_handle_array_delete_fragments_list_request(
         ctx_,
         array,
