@@ -101,8 +101,7 @@ struct SerializationFx {
     REQUIRE(stats != nullptr);
     auto counters = stats->counters();
     REQUIRE(counters != nullptr);
-    auto loop_num =
-        counters->find("Context.StorageManager.Query.Reader.loop_num");
+    auto loop_num = counters->find("Context.Query.Reader.loop_num");
     REQUIRE((loop_num != counters->end()));
     REQUIRE(loop_num->second > 0);
   }
@@ -112,19 +111,34 @@ struct SerializationFx {
     REQUIRE(stats != nullptr);
     auto counters = stats->counters();
     REQUIRE(counters != nullptr);
-    auto loop_num =
-        counters->find("Context.StorageManager.Query.Writer.attr_num");
+    auto loop_num = counters->find("Context.Query.Writer.attr_num");
     REQUIRE((loop_num != counters->end()));
     REQUIRE(loop_num->second > 0);
   }
 
+  static void check_subarray_stats(int dim0_expected, int dim1_expected) {
+    Stats::enable();
+    std::string stats;
+    Stats::dump(&stats);
+    // Note: if these checks fail, use Stats::dump(stdout) to validate counters
+    CHECK(
+        stats.find(
+            "\"Context.subSubarray.add_range_dim_0\": " +
+            std::to_string(dim0_expected)) != std::string::npos);
+    CHECK(
+        stats.find(
+            "\"Context.subSubarray.add_range_dim_1\": " +
+            std::to_string(dim1_expected)) != std::string::npos);
+    Stats::disable();
+  }
+
   static void check_delete_stats(const Query& query) {
     auto stats = ((sm::Reader*)query.ptr()->query_->strategy())->stats();
     REQUIRE(stats != nullptr);
     auto counters = stats->counters();
     REQUIRE(counters != nullptr);
-    auto dowork_num = counters->find(
-        "Context.StorageManager.Query.Deletes.dowork.timer_count");
+    auto dowork_num =
+        counters->find("Context.Query.Deletes.dowork.timer_count");
     REQUIRE((dowork_num != counters->end()));
     REQUIRE(dowork_num->second > 0);
   }
@@ -177,7 +191,9 @@ struct SerializationFx {
 
     Array array(ctx, array_uri, TILEDB_WRITE);
     Query query(ctx, array);
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -218,8 +234,10 @@ struct SerializationFx {
 
     Array array(ctx, array_uri, TILEDB_WRITE);
     Query query(ctx, array);
-    query.add_range(0, subarray[0], subarray[1]);
-    query.add_range(1, subarray[2], subarray[3]);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.add_range(0, subarray[0], subarray[1]);
+    cppapi_subarray.add_range(1, subarray[2], subarray[3]);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -354,7 +372,9 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -363,6 +383,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -391,7 +412,9 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -405,6 +428,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -454,7 +478,9 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {3, 4, 3, 4};
 
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -463,6 +489,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -498,7 +525,9 @@ TEST_CASE_METHOD(
     std::vector<char> a3_data(60);
     std::vector<uint64_t> a3_offsets(4);
     std::vector<int32_t> subarray = {3, 4, 3, 4};
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
 
     auto set_buffers = [&](Query& q) {
       q.set_data_buffer("a1", a1);
@@ -573,7 +602,9 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -623,7 +654,9 @@ TEST_CASE_METHOD(
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
     query.set_layout(TILEDB_GLOBAL_ORDER);
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx_client, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -632,6 +665,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -665,8 +699,10 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
-    query.set_subarray(subarray);
-    query.set_coordinates(coords);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
+    query.set_data_buffer("__coords", coords);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -675,6 +711,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -697,6 +734,9 @@ TEST_CASE_METHOD(
     "[query][dense][serialization][rest]") {
   create_array(TILEDB_DENSE);
   write_dense_array_ranges();
+  if (!vfs_test_setup_.is_rest()) {
+    check_subarray_stats(1, 1);
+  }
 
   SECTION("- Read all") {
     Array array(ctx, array_uri, TILEDB_READ);
@@ -708,8 +748,10 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
-    query.add_range(0, subarray[0], subarray[1]);
-    query.add_range(1, subarray[2], subarray[3]);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.add_range(0, subarray[0], subarray[1]);
+    cppapi_subarray.add_range(1, subarray[2], subarray[3]);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -718,6 +760,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -742,8 +785,10 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {3, 4, 3, 4};
 
-    query.add_range(0, subarray[0], subarray[1]);
-    query.add_range(1, subarray[2], subarray[3]);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.add_range(0, subarray[0], subarray[1]);
+    cppapi_subarray.add_range(1, subarray[2], subarray[3]);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
@@ -752,6 +797,7 @@ TEST_CASE_METHOD(
 
     // Submit query
     query.submit();
+    REQUIRE(query.query_status() == Query::Status::COMPLETE);
 
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -775,8 +821,10 @@ TEST_CASE_METHOD(
     std::vector<char> a3_data(60);
     std::vector<uint64_t> a3_offsets(4);
     std::vector<int32_t> subarray = {3, 4, 3, 4};
-    query.add_range(0, subarray[0], subarray[1]);
-    query.add_range(1, subarray[2], subarray[3]);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.add_range(0, subarray[0], subarray[1]);
+    cppapi_subarray.add_range(1, subarray[2], subarray[3]);
+    query.set_subarray(cppapi_subarray);
 
     auto set_buffers = [&](Query& q) {
       q.set_data_buffer("a1", a1);
@@ -788,7 +836,6 @@ TEST_CASE_METHOD(
 
     // Submit initial query.
     set_buffers(query);
-    // Submit query
     query.submit();
     // The deserialized query should also include the read stats
     check_read_stats(query);
@@ -853,7 +900,9 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> a3_offsets(1000);
     std::vector<int32_t> subarray = {1, 10, 1, 10};
 
-    query.set_subarray(subarray);
+    Subarray cppapi_subarray(ctx, array);
+    cppapi_subarray.set_subarray(subarray);
+    query.set_subarray(cppapi_subarray);
     query.set_data_buffer("a1", a1);
     query.set_data_buffer("a2", a2);
     query.set_validity_buffer("a2", a2_nullable);
