@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,7 +39,6 @@
 #include "tiledb/sm/filesystem/posix.h"
 #endif
 #include "tiledb/sm/c_api/tiledb.h"
-#include "tiledb/sm/misc/utils.h"
 
 #include <iostream>
 #include <sstream>
@@ -256,11 +255,17 @@ void SparseRealFx::read_sparse_array(const std::string& path) {
   float coords_dim2[16];
   uint64_t coords_size = sizeof(coords_dim1);
   tiledb_query_t* query;
+  tiledb_subarray_t* sub;
   float subarray[] = {-180.0f, 180.0f, -90.0f, 90.0f};
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
   REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_subarray_free(&sub);
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
@@ -309,11 +314,17 @@ void SparseRealFx::read_sparse_array_next_partition_bug(
   float coords_dim2[4];
   uint64_t coords_size = sizeof(coords_dim1);
   tiledb_query_t* query;
+  tiledb_subarray_t* sub;
   float subarray[] = {-180.0f, 180.0f, -90.0f, 90.0f};
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
   REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_subarray_free(&sub);
   rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
   REQUIRE(rc == TILEDB_OK);
   rc = tiledb_query_set_data_buffer(
@@ -463,6 +474,7 @@ TEST_CASE_METHOD(
   CHECK(rc == TILEDB_OK);
 
   tiledb_query_t* query;
+  tiledb_subarray_t* sub;
   rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
   REQUIRE(rc == TILEDB_OK);
 
@@ -480,13 +492,23 @@ TEST_CASE_METHOD(
   // Check Nan
   float subarray[] = {
       -180.0f, std::numeric_limits<float>::quiet_NaN(), -90.0f, 90.0f};
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
   CHECK(rc == TILEDB_ERR);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_subarray_free(&sub);
 
   // Check infinity
   subarray[1] = std::numeric_limits<float>::infinity();
-  rc = tiledb_query_set_subarray(ctx_, query, subarray);
+  rc = tiledb_subarray_alloc(ctx_, array, &sub);
+  REQUIRE(rc == TILEDB_OK);
+  rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
   CHECK(rc == TILEDB_ERR);
+  rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+  REQUIRE(rc == TILEDB_OK);
+  tiledb_subarray_free(&sub);
 
   // Clean up
   rc = tiledb_array_close(ctx_, array);
@@ -564,11 +586,18 @@ TEST_CASE_METHOD(
     double coords_dim2[1];
     uint64_t coords_size = sizeof(coords_dim1);
     tiledb_query_t* query;
+    tiledb_subarray_t* sub;
     double subarray[] = {-180.0, -180.0, 1.0, 1.0};
 
     rc = tiledb_query_alloc(ctx_, array, TILEDB_READ, &query);
     REQUIRE(rc == TILEDB_OK);
-    rc = tiledb_query_set_subarray(ctx_, query, subarray);
+    rc = tiledb_subarray_alloc(ctx_, array, &sub);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_subarray_set_subarray(ctx_, sub, subarray);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_query_set_subarray_t(ctx_, query, sub);
+    REQUIRE(rc == TILEDB_OK);
+    tiledb_subarray_free(&sub);
     REQUIRE(rc == TILEDB_OK);
     rc = tiledb_query_set_data_buffer(ctx_, query, "a", a, &a_size);
     REQUIRE(rc == TILEDB_OK);
