@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2023 TileDB Inc.
+ * @copyright Copyright (c) 2023-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@
 
 #include "test/support/src/ast_helpers.h"
 #include "test/support/tdb_catch.h"
+#include "tiledb/api/c_api/array/array_api_internal.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/cpp_api/tiledb_experimental"
@@ -260,6 +261,22 @@ TEST_CASE_METHOD(
   check_read(qc, [](const QCSetsCell& c) { return !(c.a2 == "wilma"); });
 }
 
+TEST_CASE_METHOD(
+    CPPQueryConditionFx,
+    "IN - String With Non-Enumeration Value",
+    "[query-condition][set][non-enum-value][string]") {
+  auto type = GENERATE(
+      TestArrayType::DENSE, TestArrayType::SPARSE, TestArrayType::LEGACY);
+  auto serialize = SERIALIZE_TESTS();
+  create_array(type, serialize);
+
+  std::vector<std::string> values = {"wilma", "astro"};
+  auto qc =
+      QueryConditionExperimental::create(ctx_, "attr2", values, TILEDB_NOT_IN);
+
+  check_read(qc, [](const QCSetsCell& c) { return !(c.a2 == "wilma"); });
+}
+
 TEST_CASE_METHOD(
     CPPQueryConditionFx,
     "NOT_IN - Enumeration",
@@ -722,9 +739,8 @@ void CPPQueryConditionFx::check_read(
     qc = serialize_deserialize_qc(qc);
   }
 
-  auto core_array = array.ptr().get()->array_;
   auto core_qc = qc.ptr().get()->query_condition_;
-  throw_if_not_ok(core_qc->check(core_array->array_schema_latest()));
+  throw_if_not_ok(core_qc->check(array.ptr().get()->array_schema_latest()));
 
   query.set_condition(qc)
       .set_data_buffer("dim", dim_read)
@@ -828,7 +844,7 @@ void CPPQueryConditionFx::rm_array() {
 }
 
 void CPPQueryConditionFx::generate_data() {
-  num_elements_ = 10;  // * 1024;
+  num_elements_ = 1024;
 
   dim_values_.clear();
   attr1_values_.clear();
@@ -924,7 +940,7 @@ QueryCondition CPPQueryConditionFx::serialize_deserialize_qc(
   throw_if_not_ok(condition_to_capnp(*qc_ptr, &builder));
 
   // Deserialize the query condition.
-  throw_if_not_ok(condition_from_capnp(builder, ret_ptr));
+  *ret_ptr = condition_from_capnp(builder);
   REQUIRE(tiledb::test::ast_equal(ret_ptr->ast(), qc_ptr->ast()));
 
   return ret;
@@ -1012,7 +1028,8 @@ std::vector<std::string> CPPQueryConditionFx::to_vector(
 template <typename T>
 T CPPQueryConditionFx::choose_value(std::vector<T>& values) {
   auto rval = random();
-  auto idx = static_cast<size_t>(rval * values.size());
+  // Note the `% values.size()` which handles when rval is 1.0
+  auto idx = static_cast<size_t>(rval * values.size()) % values.size();
   return values[idx];
 }
 
