@@ -138,7 +138,8 @@ std::string local_path() {
 TEST_CASE("VFS: Test long local paths", "[vfs]") {
   ThreadPool compute_tp(4);
   ThreadPool io_tp(4);
-  VFS vfs{&g_helper_stats, &compute_tp, &io_tp, Config{}};
+  VFS vfs{
+      &g_helper_stats, g_helper_logger().get(), &compute_tp, &io_tp, Config{}};
 
   SECTION("- Deep hierarchy") {
     // Create a nested path with a long total length
@@ -209,7 +210,8 @@ TEMPLATE_LIST_TEST_CASE(
   ThreadPool compute_tp(4);
   ThreadPool io_tp(4);
   Config config = set_config_params();
-  VFS vfs{&g_helper_stats, &compute_tp, &io_tp, config};
+  VFS vfs{
+      &g_helper_stats, g_helper_logger().get(), &compute_tp, &io_tp, config};
 
   URI path = fs.temp_dir_.add_trailing_slash();
 
@@ -339,9 +341,7 @@ TEMPLATE_LIST_TEST_CASE(
     std::string s = "abcdef";
     require_tiledb_ok(vfs.write(ls_file, s.data(), s.size()));
     require_tiledb_ok(vfs.close_file(ls_file));
-    auto&& [status, opt_children] = vfs.ls_with_sizes(ls_dir);
-    require_tiledb_ok(status);
-    auto children = opt_children.value();
+    auto children = vfs.ls_with_sizes(ls_dir);
 #ifdef _WIN32
     // Normalization only for Windows
     ls_file = URI(tiledb::sm::path_win::uri_from_path(ls_file.to_string()));
@@ -445,7 +445,8 @@ TEMPLATE_LIST_TEST_CASE("VFS: File I/O", "[vfs][uri][file_io]", AllBackends) {
   ThreadPool compute_tp(4);
   ThreadPool io_tp(4);
   Config config = set_config_params(disable_multipart, max_parallel_ops);
-  VFS vfs{&g_helper_stats, &compute_tp, &io_tp, config};
+  VFS vfs{
+      &g_helper_stats, g_helper_logger().get(), &compute_tp, &io_tp, config};
 
   // Getting file_size on a nonexistent blob shouldn't crash on Azure
   uint64_t nbytes = 0;
@@ -564,7 +565,8 @@ TEST_CASE("VFS: Test end-to-end", "[.vfs-e2e]") {
   // Will be configured from environment variables.
   Config config;
 
-  VFS vfs{&g_helper_stats, &compute_tp, &io_tp, config};
+  VFS vfs{
+      &g_helper_stats, g_helper_logger().get(), &compute_tp, &io_tp, config};
   REQUIRE(vfs.supports_uri_scheme(test_file));
 
   uint64_t nbytes = 0;
@@ -575,7 +577,8 @@ TEST_CASE("VFS: Test end-to-end", "[.vfs-e2e]") {
 TEST_CASE("VFS: test ls_with_sizes", "[vfs][ls-with-sizes]") {
   ThreadPool compute_tp(4);
   ThreadPool io_tp(4);
-  VFS vfs_ls{&g_helper_stats, &compute_tp, &io_tp, Config{}};
+  VFS vfs_ls{
+      &g_helper_stats, g_helper_logger().get(), &compute_tp, &io_tp, Config{}};
 
   std::string path = local_path();
   std::string dir = path + "ls_dir";
@@ -599,9 +602,7 @@ TEST_CASE("VFS: test ls_with_sizes", "[vfs][ls-with-sizes]") {
   require_tiledb_ok(vfs_ls.write(URI(subdir_file), s2.data(), s2.size()));
 
   // List
-  auto&& [status, rv] = vfs_ls.ls_with_sizes(URI(dir));
-  auto children = *rv;
-  require_tiledb_ok(status);
+  auto children = vfs_ls.ls_with_sizes(URI(dir));
 
 #ifdef _WIN32
   // Normalization only for Windows
@@ -622,8 +623,9 @@ TEST_CASE("VFS: test ls_with_sizes", "[vfs][ls-with-sizes]") {
   require_tiledb_ok(vfs_ls.remove_dir(URI(path)));
 }
 
-// Currently only S3 is supported for VFS::ls_recursive.
-using TestBackends = std::tuple<S3Test>;
+// Currently only local, S3, Azure and GCS are supported for VFS::ls_recursive.
+// TODO: LocalFsTest currently fails. Fix and re-enable.
+using TestBackends = std::tuple</*LocalFsTest,*/ S3Test, AzureTest, GCSTest>;
 TEMPLATE_LIST_TEST_CASE(
     "VFS: Test internal ls_filtered recursion argument",
     "[vfs][ls_filtered][recursion]",
@@ -637,10 +639,9 @@ TEMPLATE_LIST_TEST_CASE(
   DYNAMIC_SECTION(
       fs.temp_dir_.backend_name()
       << " ls_filtered with recursion: " << (recursive ? "true" : "false")) {
-#ifdef HAVE_S3
     // If testing with recursion use the root directory, otherwise use a subdir.
     auto path = recursive ? fs.temp_dir_ : fs.temp_dir_.join_path("subdir_1");
-    auto ls_objects = fs.get_s3().ls_filtered(
+    auto ls_objects = fs.vfs_.ls_filtered(
         path, VFSTestBase::accept_all_files, accept_all_dirs, recursive);
 
     auto expected = fs.expected_results();
@@ -654,7 +655,6 @@ TEMPLATE_LIST_TEST_CASE(
 
     CHECK(ls_objects.size() == expected.size());
     CHECK(ls_objects == expected);
-#endif
   }
 }
 
@@ -669,12 +669,7 @@ TEST_CASE(
   }
   std::string backend = vfs_test.temp_dir_.backend_name();
 
-  if (vfs_test.temp_dir_.is_s3()) {
-    DYNAMIC_SECTION(backend << " supported backend should not throw") {
-      CHECK_NOTHROW(vfs_test.vfs_.ls_recursive(
-          vfs_test.temp_dir_, VFSTestBase::accept_all_files));
-    }
-  } else {
+  if (vfs_test.temp_dir_.is_hdfs()) {
     DYNAMIC_SECTION(backend << " unsupported backend should throw") {
       CHECK_THROWS_WITH(
           vfs_test.vfs_.ls_recursive(
@@ -682,13 +677,18 @@ TEST_CASE(
           Catch::Matchers::ContainsSubstring(
               "storage backend is not supported"));
     }
+  } else {
+    DYNAMIC_SECTION(backend << " supported backend should not throw") {
+      CHECK_NOTHROW(vfs_test.vfs_.ls_recursive(
+          vfs_test.temp_dir_, VFSTestBase::accept_all_files));
+    }
   }
 }
 
 TEST_CASE(
     "VFS: Throwing FileFilter for ls_recursive",
     "[vfs][ls_recursive][file-filter]") {
-  std::string prefix = "s3://";
+  std::string prefix = GENERATE("s3://", "azure://", "gcs://", "gs://");
   VFSTest vfs_test({0}, prefix);
   if (!vfs_test.is_supported()) {
     return;
@@ -712,39 +712,80 @@ TEST_CASE(
   }
 }
 
+TEST_CASE("VFS: Test remove_dir_if_empty", "[vfs][remove-dir-if-empty]") {
+  ThreadPool tp(1);
+  VFS vfs{&g_helper_stats, g_helper_logger().get(), &tp, &tp, Config{}};
+
+  std::string path = local_path();
+  std::string dir = path + "remove_dir_if_empty/";
+  std::string subdir = dir + "subdir/";
+  std::string file1 = dir + "file1";
+
+  // Create directories and files
+  require_tiledb_ok(vfs.create_dir(URI(path)));
+  require_tiledb_ok(vfs.create_dir(URI(dir)));
+  require_tiledb_ok(vfs.create_dir(URI(subdir)));
+  require_tiledb_ok(vfs.touch(URI(file1)));
+
+  // Check that remove_dir_if_empty fails for non-empty directories
+  vfs.remove_dir_if_empty(URI(dir));
+  bool exists;
+  require_tiledb_ok(vfs.is_dir(URI(dir), &exists));
+  CHECK(exists);
+
+  // Check that it succeeds for empty directories
+  vfs.remove_dir_if_empty(URI(subdir));
+  require_tiledb_ok(vfs.is_dir(URI(subdir), &exists));
+  CHECK_FALSE(exists);
+
+  // Empty the directory and try again
+  require_tiledb_ok(vfs.remove_file(URI(file1)));
+  vfs.remove_dir_if_empty(URI(dir));
+  require_tiledb_ok(vfs.is_dir(URI(dir), &exists));
+  CHECK_FALSE(exists);
+
+  // Clean up
+  require_tiledb_ok(vfs.remove_dir(URI(path)));
+}
+
 #ifdef HAVE_AZURE
 TEST_CASE("VFS: Construct Azure Blob Storage endpoint URIs", "[azure][uri]") {
+  // Test the construction of Azure Blob Storage URIs from account name and SAS
+  // token. We are not actually connecting to Azure Blob Storage in this test.
   std::string sas_token, custom_endpoint, expected_endpoint;
   SECTION("No SAS token") {
     sas_token = "";
-    expected_endpoint = "https://devstoreaccount1.blob.core.windows.net";
+    expected_endpoint = "https://exampleaccount.blob.core.windows.net";
   }
   SECTION("SAS token without leading question mark") {
     sas_token = "baz=qux&foo=bar";
     expected_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
   }
   SECTION("SAS token with leading question mark") {
     sas_token = "?baz=qux&foo=bar";
     expected_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
   }
   SECTION("SAS token in both endpoint and config option") {
     sas_token = "baz=qux&foo=bar";
     custom_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
     expected_endpoint =
-        "https://devstoreaccount1.blob.core.windows.net?baz=qux&foo=bar";
-  }
-  SECTION("No SAS token") {
-    sas_token = "";
-    expected_endpoint = "https://devstoreaccount1.blob.core.windows.net";
+        "https://exampleaccount.blob.core.windows.net?baz=qux&foo=bar";
   }
   Config config;
   require_tiledb_ok(
-      config.set("vfs.azure.storage_account_name", "devstoreaccount1"));
+      config.set("vfs.azure.storage_account_name", "exampleaccount"));
   require_tiledb_ok(config.set("vfs.azure.blob_endpoint", custom_endpoint));
   require_tiledb_ok(config.set("vfs.azure.storage_sas_token", sas_token));
+  if (sas_token.empty()) {
+    // If the SAS token is empty, the VFS will try to connect to Microsoft Entra
+    // ID to obtain credentials, which can take a long time because of retries.
+    // Set a dummy access key (which won't be used because we are not going to
+    // perform any requests) to prevent Entra ID from being chosen.
+    require_tiledb_ok(config.set("vfs.azure.storage_account_key", "foobar"));
+  }
   tiledb::sm::Azure azure;
   ThreadPool thread_pool(1);
   require_tiledb_ok(azure.init(config, &thread_pool));
