@@ -32,6 +32,7 @@
  **/
 
 #include "tiledb/api/c_api_support/c_api_support.h"
+#include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/c_api/tiledb_serialization.h"
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/group/group_details_v1.h"
@@ -65,6 +66,12 @@ inline void ensure_name_argument_is_valid(const char* name) {
   }
 }
 
+inline void ensure_object_type_argument_is_valid(const tiledb_object_t type) {
+  if (type != TILEDB_GROUP && type != TILEDB_ARRAY) {
+    throw CAPIStatusException("Invalid object `type`");
+  }
+}
+
 inline char* copy_string(const std::string& str) {
   auto ret = static_cast<char*>(std::malloc(str.size() + 1));
   if (ret == nullptr) {
@@ -81,7 +88,7 @@ capi_return_t tiledb_group_create(
     tiledb_ctx_handle_t* ctx, const char* group_uri) {
   ensure_group_uri_argument_is_valid(group_uri);
 
-  throw_if_not_ok(ctx->storage_manager()->group_create(group_uri));
+  tiledb::sm::Group::create(ctx->resources(), tiledb::sm::URI(group_uri));
 
   return TILEDB_OK;
 }
@@ -99,7 +106,7 @@ capi_return_t tiledb_group_alloc(
         "Failed to allocate TileDB group API object; Invalid URI");
   }
 
-  *group = tiledb_group_handle_t::make_handle(uri, ctx->storage_manager());
+  *group = tiledb_group_handle_t::make_handle(ctx->resources(), uri);
 
   return TILEDB_OK;
 }
@@ -114,8 +121,7 @@ capi_return_t tiledb_group_open(
     tiledb_group_handle_t* group, tiledb_query_type_t query_type) {
   ensure_group_is_valid(group);
 
-  throw_if_not_ok(
-      group->group().open(static_cast<tiledb::sm::QueryType>(query_type)));
+  group->group().open(static_cast<tiledb::sm::QueryType>(query_type));
 
   return TILEDB_OK;
 }
@@ -123,7 +129,7 @@ capi_return_t tiledb_group_open(
 capi_return_t tiledb_group_close(tiledb_group_handle_t* group) {
   ensure_group_is_valid(group);
 
-  throw_if_not_ok(group->group().close());
+  group->group().close();
 
   return TILEDB_OK;
 }
@@ -270,8 +276,31 @@ capi_return_t tiledb_group_add_member(
     name_optional = name;
   }
 
-  throw_if_not_ok(
-      group->group().mark_member_for_addition(uri, relative, name_optional));
+  group->group().mark_member_for_addition(
+      uri, relative, name_optional, std::nullopt);
+
+  return TILEDB_OK;
+}
+
+capi_return_t tiledb_group_add_member_with_type(
+    tiledb_group_handle_t* group,
+    const char* group_uri,
+    const uint8_t relative,
+    const char* name,
+    tiledb_object_t type) {
+  ensure_group_is_valid(group);
+  ensure_group_uri_argument_is_valid(group_uri);
+  ensure_object_type_argument_is_valid(type);
+
+  auto uri = tiledb::sm::URI(group_uri, !relative);
+
+  std::optional<std::string> name_optional = std::nullopt;
+  if (name != nullptr) {
+    name_optional = name;
+  }
+
+  group->group().mark_member_for_addition(
+      uri, relative, name_optional, static_cast<tiledb::sm::ObjectType>(type));
 
   return TILEDB_OK;
 }
@@ -281,7 +310,7 @@ capi_return_t tiledb_group_remove_member(
   ensure_group_is_valid(group);
   ensure_name_argument_is_valid(name_or_uri);
 
-  throw_if_not_ok(group->group().mark_member_for_removal(name_or_uri));
+  group->group().mark_member_for_removal(name_or_uri);
 
   return TILEDB_OK;
 }
@@ -296,64 +325,12 @@ capi_return_t tiledb_group_get_member_count(
   return TILEDB_OK;
 }
 
-capi_return_t tiledb_group_get_member_by_index(
-    tiledb_group_handle_t* group,
-    uint64_t index,
-    char** uri,
-    tiledb_object_t* type,
-    char** name) {
-  ensure_group_is_valid(group);
-  ensure_output_pointer_is_valid(uri);
-  ensure_output_pointer_is_valid(type);
-  ensure_output_pointer_is_valid(name);
-
-  LOG_WARN(
-      "tiledb_group_get_member_by_index is deprecated. Please use "
-      "tiledb_group_get_member_by_index_v2 instead.");
-
-  char* tmp_uri = nullptr;
-  char* tmp_name = nullptr;
-
-  auto&& [uri_str, object_type, name_str] =
-      group->group().member_by_index(index);
-
-  tmp_uri = copy_string(uri_str);
-  if (tmp_uri == nullptr) {
-    goto error;
-  }
-
-  if (name_str.has_value()) {
-    tmp_name = copy_string(name_str.value());
-    if (tmp_name == nullptr) {
-      goto error;
-    }
-  }
-
-  *uri = tmp_uri;
-  *type = static_cast<tiledb_object_t>(object_type);
-  *name = tmp_name;
-
-  return TILEDB_OK;
-
-error:
-
-  if (tmp_uri != nullptr) {
-    std::free(tmp_uri);
-  }
-
-  if (tmp_name != nullptr) {
-    std::free(tmp_name);
-  }
-
-  return TILEDB_OK;
-}
-
 capi_return_t tiledb_group_get_member_by_index_v2(
     tiledb_group_handle_t* group,
     uint64_t index,
-    tiledb_string_t** uri,
+    tiledb_string_handle_t** uri,
     tiledb_object_t* type,
-    tiledb_string_t** name) {
+    tiledb_string_handle_t** name) {
   ensure_group_is_valid(group);
   ensure_output_pointer_is_valid(uri);
   ensure_output_pointer_is_valid(type);
@@ -376,37 +353,10 @@ capi_return_t tiledb_group_get_member_by_index_v2(
   return TILEDB_OK;
 }
 
-capi_return_t tiledb_group_get_member_by_name(
-    tiledb_group_handle_t* group,
-    const char* name,
-    char** uri,
-    tiledb_object_t* type) {
-  ensure_group_is_valid(group);
-  ensure_name_argument_is_valid(name);
-  ensure_output_pointer_is_valid(uri);
-  ensure_output_pointer_is_valid(type);
-
-  LOG_WARN(
-      "tiledb_group_get_member_by_name is deprecated. Please use "
-      "tiledb_group_get_member_by_name_v2 instead.");
-
-  auto&& [uri_str, object_type, name_str, ignored_relative] =
-      group->group().member_by_name(name);
-
-  *uri = copy_string(uri_str);
-  if (*uri == nullptr) {
-    return TILEDB_ERR;
-  }
-
-  *type = static_cast<tiledb_object_t>(object_type);
-
-  return TILEDB_OK;
-}
-
 capi_return_t tiledb_group_get_member_by_name_v2(
     tiledb_group_handle_t* group,
     const char* name,
-    tiledb_string_t** uri,
+    tiledb_string_handle_t** uri,
     tiledb_object_t* type) {
   ensure_group_is_valid(group);
   ensure_output_pointer_is_valid(uri);
@@ -463,8 +413,7 @@ capi_return_t tiledb_group_get_query_type(
   ensure_output_pointer_is_valid(query_type);
 
   // Get query_type
-  tiledb::sm::QueryType type;
-  throw_if_not_ok(group->group().get_query_type(&type));
+  tiledb::sm::QueryType type = group->group().get_query_type();
 
   *query_type = static_cast<tiledb_query_type_t>(type);
 
@@ -485,7 +434,21 @@ capi_return_t tiledb_group_dump_str(
   return TILEDB_OK;
 }
 
+capi_return_t tiledb_group_dump_str_v2(
+    tiledb_group_handle_t* group,
+    tiledb_string_handle_t** dump_ascii,
+    const uint8_t recursive) {
+  ensure_group_is_valid(group);
+  ensure_output_pointer_is_valid(dump_ascii);
+
+  *dump_ascii =
+      tiledb_string_handle_t::make_handle(group->group().dump(2, 0, recursive));
+
+  return TILEDB_OK;
+}
+
 capi_return_t tiledb_serialize_group(
+    tiledb_ctx_handle_t* ctx,
     const tiledb_group_handle_t* group,
     tiledb_serialization_type_t serialize_type,
     int32_t,
@@ -494,14 +457,15 @@ capi_return_t tiledb_serialize_group(
   ensure_output_pointer_is_valid(buffer);
 
   // ALERT: Conditional Handle Construction
-  auto buf = tiledb_buffer_handle_t::make_handle();
+  auto buf = tiledb_buffer_handle_t::make_handle(
+      ctx->resources().serialization_memory_tracker());
 
   // We're not using throw_if_not_ok here because we have to
   // clean up our allocated buffer if serialization fails.
   auto st = tiledb::sm::serialization::group_serialize(
       &(group->group()),
       static_cast<tiledb::sm::SerializationType>(serialize_type),
-      &(buf->buffer()));
+      buf->buffer());
 
   if (!st.ok()) {
     tiledb_buffer_handle_t::break_handle(buf);
@@ -530,6 +494,7 @@ capi_return_t tiledb_deserialize_group(
 }
 
 capi_return_t tiledb_serialize_group_metadata(
+    tiledb_ctx_handle_t* ctx,
     const tiledb_group_handle_t* group,
     tiledb_serialization_type_t serialize_type,
     tiledb_buffer_t** buffer) {
@@ -537,7 +502,8 @@ capi_return_t tiledb_serialize_group_metadata(
   ensure_output_pointer_is_valid(buffer);
 
   // ALERT: Conditional Handle Construction
-  auto buf = tiledb_buffer_handle_t::make_handle();
+  auto buf = tiledb_buffer_handle_t::make_handle(
+      ctx->resources().serialization_memory_tracker());
 
   // Get metadata to serialize, this will load it if it does not exist
   auto metadata = group->group().metadata();
@@ -545,7 +511,7 @@ capi_return_t tiledb_serialize_group_metadata(
   auto st = tiledb::sm::serialization::metadata_serialize(
       metadata,
       static_cast<tiledb::sm::SerializationType>(serialize_type),
-      &(buf->buffer()));
+      buf->buffer());
 
   if (!st.ok()) {
     tiledb_buffer_handle_t::break_handle(buf);
@@ -566,6 +532,7 @@ capi_return_t tiledb_deserialize_group_metadata(
 
   throw_if_not_ok(tiledb::sm::serialization::metadata_deserialize(
       group->group().unsafe_metadata(),
+      group->group().config(),
       static_cast<tiledb::sm::SerializationType>(serialize_type),
       buffer->buffer()));
 
@@ -576,10 +543,8 @@ capi_return_t tiledb_group_consolidate_metadata(
     tiledb_ctx_handle_t* ctx, const char* group_uri, tiledb_config_t* config) {
   ensure_group_uri_argument_is_valid(group_uri);
 
-  auto cfg =
-      (config == nullptr) ? ctx->storage_manager()->config() : config->config();
-  throw_if_not_ok(
-      ctx->storage_manager()->group_metadata_consolidate(group_uri, cfg));
+  auto cfg = (config == nullptr) ? ctx->config() : config->config();
+  tiledb::sm::Group::consolidate_metadata(ctx->resources(), group_uri, cfg);
 
   return TILEDB_OK;
 }
@@ -588,9 +553,8 @@ capi_return_t tiledb_group_vacuum_metadata(
     tiledb_ctx_handle_t* ctx, const char* group_uri, tiledb_config_t* config) {
   ensure_group_uri_argument_is_valid(group_uri);
 
-  auto cfg =
-      (config == nullptr) ? ctx->storage_manager()->config() : config->config();
-  ctx->storage_manager()->group_metadata_vacuum(group_uri, cfg);
+  auto cfg = (config == nullptr) ? ctx->config() : config->config();
+  sm::Group::vacuum_metadata(ctx->resources(), group_uri, cfg);
 
   return TILEDB_OK;
 }
@@ -738,6 +702,18 @@ CAPI_INTERFACE(
       ctx, group, uri, relative, name);
 }
 
+CAPI_INTERFACE(
+    group_add_member_with_type,
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    const char* uri,
+    const uint8_t relative,
+    const char* name,
+    tiledb_object_t type) {
+  return api_entry_context<tiledb::api::tiledb_group_add_member_with_type>(
+      ctx, group, uri, relative, name, type);
+}
+
 CAPI_INTERFACE(
     group_remove_member,
     tiledb_ctx_t* ctx,
@@ -756,47 +732,24 @@ CAPI_INTERFACE(
       ctx, group, count);
 }
 
-CAPI_INTERFACE(
-    group_get_member_by_index,
-    tiledb_ctx_t* ctx,
-    tiledb_group_t* group,
-    uint64_t index,
-    char** uri,
-    tiledb_object_t* type,
-    char** name) {
-  return api_entry_context<tiledb::api::tiledb_group_get_member_by_index>(
-      ctx, group, index, uri, type, name);
-}
-
 CAPI_INTERFACE(
     group_get_member_by_index_v2,
     tiledb_ctx_t* ctx,
     tiledb_group_t* group,
     uint64_t index,
-    tiledb_string_t** uri,
+    tiledb_string_handle_t** uri,
     tiledb_object_t* type,
-    tiledb_string_t** name) {
+    tiledb_string_handle_t** name) {
   return api_entry_context<tiledb::api::tiledb_group_get_member_by_index_v2>(
       ctx, group, index, uri, type, name);
 }
 
-CAPI_INTERFACE(
-    group_get_member_by_name,
-    tiledb_ctx_t* ctx,
-    tiledb_group_t* group,
-    const char* name,
-    char** uri,
-    tiledb_object_t* type) {
-  return api_entry_context<tiledb::api::tiledb_group_get_member_by_name>(
-      ctx, group, name, uri, type);
-}
-
 CAPI_INTERFACE(
     group_get_member_by_name_v2,
     tiledb_ctx_t* ctx,
     tiledb_group_t* group,
     const char* name,
-    tiledb_string_t** uri,
+    tiledb_string_handle_t** uri,
     tiledb_object_t* type) {
   return api_entry_context<tiledb::api::tiledb_group_get_member_by_name_v2>(
       ctx, group, name, uri, type);
@@ -847,6 +800,16 @@ CAPI_INTERFACE(
       ctx, group, dump_ascii, recursive);
 }
 
+CAPI_INTERFACE(
+    group_dump_str_v2,
+    tiledb_ctx_t* ctx,
+    tiledb_group_t* group,
+    tiledb_string_handle_t** dump_ascii,
+    const uint8_t recursive) {
+  return api_entry_context<tiledb::api::tiledb_group_dump_str_v2>(
+      ctx, group, dump_ascii, recursive);
+}
+
 CAPI_INTERFACE(
     serialize_group,
     tiledb_ctx_t* ctx,
@@ -854,7 +817,7 @@ CAPI_INTERFACE(
     tiledb_serialization_type_t serialize_type,
     int32_t client_side,
     tiledb_buffer_t** buffer_list) {
-  return api_entry_context<tiledb::api::tiledb_serialize_group>(
+  return api_entry_with_context<tiledb::api::tiledb_serialize_group>(
       ctx, group, serialize_type, client_side, buffer_list);
 }
 
@@ -875,7 +838,7 @@ CAPI_INTERFACE(
     const tiledb_group_t* group,
     tiledb_serialization_type_t serialization_type,
     tiledb_buffer_t** buffer) {
-  return api_entry_context<tiledb::api::tiledb_serialize_group_metadata>(
+  return api_entry_with_context<tiledb::api::tiledb_serialize_group_metadata>(
       ctx, group, serialization_type, buffer);
 }
 
