@@ -230,7 +230,10 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 9, 1, 2};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(tiledb_subarray_set_subarray(ctx, subarray, &dom) == TILEDB_OK);
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   // nullptr context
   tiledb_query_channel_t* default_channel = nullptr;
@@ -324,6 +327,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -340,7 +344,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 9, 1, 2};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -365,6 +378,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -381,7 +395,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 10, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -411,6 +434,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -427,7 +451,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 10, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -459,6 +492,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -475,7 +509,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 10, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -506,6 +549,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -522,7 +566,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 10, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -553,6 +606,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -569,7 +623,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 10, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -601,6 +664,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -617,7 +681,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 9, 1, 2};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -654,6 +727,7 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -668,7 +742,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_alloc(ctx, array, TILEDB_READ, &query) == TILEDB_OK);
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
   int64_t dom[] = {1, 2, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   std::vector<int32_t> d(4);
   uint64_t size = 1;
@@ -697,6 +780,7 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
 
 TEST_CASE_METHOD(
@@ -713,7 +797,16 @@ TEST_CASE_METHOD(
   REQUIRE(tiledb_query_set_layout(ctx, query, TILEDB_UNORDERED) == TILEDB_OK);
 
   int64_t dom[] = {1, 10, 1, 1};
-  REQUIRE(tiledb_query_set_subarray(ctx, query, &dom) == TILEDB_OK);
+  tiledb_subarray_t* subarray;
+  REQUIRE(tiledb_subarray_alloc(ctx, array, &subarray) == TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 0, &dom[0], &dom[1], nullptr) ==
+      TILEDB_OK);
+  REQUIRE(
+      tiledb_subarray_add_range(ctx, subarray, 1, &dom[2], &dom[3], nullptr) ==
+      TILEDB_OK);
+
+  REQUIRE(tiledb_query_set_subarray_t(ctx, query, subarray) == TILEDB_OK);
 
   tiledb_query_channel_t* default_channel;
   REQUIRE(
@@ -751,4 +844,5 @@ TEST_CASE_METHOD(
   CHECK(tiledb_array_close(ctx, array) == TILEDB_OK);
   tiledb_array_free(&array);
   tiledb_query_free(&query);
+  tiledb_subarray_free(&subarray);
 }
