@@ -32,6 +32,7 @@
 #include <test/support/tdb_catch.h>
 #include "../vfs_api_internal.h"
 #include "tiledb/api/c_api_test_support/testsupport_capi_vfs.h"
+#include "tiledb/platform/platform.h"
 
 using tiledb::api::test_support::ordinary_vfs;
 using tiledb::api::test_support::ordinary_vfs_fh;
@@ -246,6 +247,12 @@ TEST_CASE("C API: tiledb_vfs_is_dir argument validation", "[capi][vfs]") {
     CHECK(tiledb_status(rc) == TILEDB_ERR);
   }
   SECTION("null uri") {
+    if constexpr (tiledb::platform::is_os_windows) {
+      // Windows handles empty (which gets converted from null) paths
+      // differently. Reconsider when the logic gets unified across paltforms
+      // (SC-28225).
+      return;
+    }
     auto rc{tiledb_vfs_is_dir(x.ctx, x.vfs, nullptr, &is_dir)};
     CHECK(tiledb_status(rc) == TILEDB_ERR);
   }
@@ -331,6 +338,10 @@ TEST_CASE("C API: tiledb_vfs_move_dir argument validation", "[capi][vfs]") {
 }
 
 TEST_CASE("C API: tiledb_vfs_copy_dir argument validation", "[capi][vfs]") {
+  if constexpr (tiledb::platform::is_os_windows) {
+    // This API is not yet supported on Windows.
+    return;
+  }
   ordinary_vfs x;
   SECTION("success") {
     auto rc{tiledb_vfs_copy_dir(x.ctx, x.vfs, TEST_URI, "new_dir")};
@@ -412,6 +423,12 @@ TEST_CASE("C API: tiledb_vfs_is_file argument validation", "[capi][vfs]") {
     CHECK(tiledb_status(rc) == TILEDB_ERR);
   }
   SECTION("null uri") {
+    if constexpr (tiledb::platform::is_os_windows) {
+      // Windows handles empty (which gets converted from null) paths
+      // differently. Reconsider when the logic gets unified across paltforms
+      // (SC-28225).
+      return;
+    }
     auto rc{tiledb_vfs_is_file(x.ctx, x.vfs, nullptr, &is_file)};
     CHECK(tiledb_status(rc) == TILEDB_ERR);
   }
@@ -477,6 +494,10 @@ TEST_CASE("C API: tiledb_vfs_move_file argument validation", "[capi][vfs]") {
 }
 
 TEST_CASE("C API: tiledb_vfs_copy_file argument validation", "[capi][vfs]") {
+  if constexpr (tiledb::platform::is_os_windows) {
+    // This API is not yet supported on Windows.
+    return;
+  }
   ordinary_vfs x;
   SECTION("success") {
     auto rc{tiledb_vfs_copy_file(x.ctx, x.vfs, TEST_URI, "new_uri")};
