@@ -58,6 +58,8 @@ class Buffer;
 class ConstBuffer;
 class Dimension;
 class DomainTypedDataView;
+class CurrentDomain;
+class NDRectangle;
 class FilterPipeline;
 class MemoryTracker;
 enum class Datatype : uint8_t;
@@ -227,7 +229,7 @@ class Domain {
    * @return non-null pointer to the dimension
    */
   inline const Dimension* dimension_ptr(dimension_size_type i) const {
-    if (i > dim_num_) {
+    if (i >= dim_num_) {
       throw std::invalid_argument("invalid dimension index");
     }
     return dimension_ptrs_[i];
@@ -269,9 +271,6 @@ class Domain {
    */
   shared_ptr<Dimension> shared_dimension(const std::string& name) const;
 
-  /** Dumps the domain in ASCII format in the selected output. */
-  void dump(FILE* out) const;
-
   /** Expands ND range `r2` using ND range `r1`. */
   void expand_ndrange(const NDRange& r1, NDRange* r2) const;
 
@@ -280,8 +279,10 @@ class Domain {
    * the array's regular tiles (i.e., it maps it on the regular tile grid).
    * If the array has no regular tile grid or real domain, the function
    * does not do anything.
+   *
+   * @param query_ndrange The query domain to be expanded.
    */
-  void expand_to_tiles(NDRange* ndrange) const;
+  void expand_to_tiles_when_no_current_domain(NDRange& query_ndrange) const;
 
   /**
    * Retrieves the tile coordinates of the input cell coordinates.
@@ -702,4 +703,7 @@ class Domain {
 
 }  // namespace tiledb::sm
 
+/** Converts the filter into a string representation. */
+std::ostream& operator<<(std::ostream& os, const tiledb::sm::Domain& domain);
+
 #endif  // TILEDB_DOMAIN_H
