@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,6 @@
 #include "tiledb/sm/filesystem/azure.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 
 static std::shared_ptr<::Azure::Core::Http::HttpTransport> create_transport(
     const tiledb::sm::SSLConfig& ssl_cfg);
@@ -54,8 +53,19 @@ static std::shared_ptr<::Azure::Core::Http::HttpTransport> create_transport(
 using namespace tiledb::common;
 using tiledb::common::filesystem::directory_entry;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
+
+/** Converts an Azure nullable value to an STL optional. */
+template <class T>
+std::optional<T> from_azure_nullable(const ::Azure::Nullable<T>& value) {
+  return value ? std::optional(*value) : nullopt;
+}
+
+/** Converts an STL optional value to an Azure nullable. */
+template <class T>
+::Azure::Nullable<T> to_azure_nullable(const std::optional<T>& value) {
+  return value ? *value : ::Azure::Nullable<T>{};
+}
 
 /* ********************************* */
 /*     CONSTRUCTORS & DESTRUCTORS    */
@@ -542,7 +552,7 @@ Status Azure::is_blob(
   return Status_Ok();
 }
 
-std::string Azure::remove_front_slash(const std::string& path) const {
+std::string Azure::remove_front_slash(const std::string& path) {
   if (path.front() == '/') {
     return path.substr(1, path.length());
   }
@@ -550,7 +560,7 @@ std::string Azure::remove_front_slash(const std::string& path) const {
   return path;
 }
 
-std::string Azure::add_trailing_slash(const std::string& path) const {
+std::string Azure::add_trailing_slash(const std::string& path) {
   if (path.back() != '/') {
     return path + "/";
   }
@@ -558,7 +568,7 @@ std::string Azure::add_trailing_slash(const std::string& path) const {
   return path;
 }
 
-std::string Azure::remove_trailing_slash(const std::string& path) const {
+std::string Azure::remove_trailing_slash(const std::string& path) {
   if (path.back() == '/') {
     return path.substr(0, path.length() - 1);
   }
@@ -573,32 +583,26 @@ Status Azure::ls(
     const int max_paths) const {
   assert(paths);
 
-  auto&& [st, entries] = ls_with_sizes(uri, delimiter, max_paths);
-  RETURN_NOT_OK(st);
-
-  for (auto& fs : *entries) {
+  for (auto& fs : ls_with_sizes(uri, delimiter, max_paths)) {
     paths->emplace_back(fs.path().native());
   }
 
   return Status::Ok();
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> Azure::ls_with_sizes(
+std::vector<directory_entry> Azure::ls_with_sizes(
     const URI& uri, const std::string& delimiter, int max_paths) const {
   const auto& c = client();
 
   const URI uri_dir = uri.add_trailing_slash();
 
   if (!uri_dir.is_azure()) {
-    auto st = LOG_STATUS(Status_AzureError(
-        std::string("URI is not an Azure URI: " + uri_dir.to_string())));
-    return {st, nullopt};
+    throw AzureException("URI is not an Azure URI: " + uri_dir.to_string());
   }
 
   std::string container_name;
   std::string blob_path;
-  RETURN_NOT_OK_TUPLE(
-      parse_azure_uri(uri_dir, &container_name, &blob_path), nullopt);
+  throw_if_not_ok(parse_azure_uri(uri_dir, &container_name, &blob_path));
 
   auto container_client = c.GetBlobContainerClient(container_name);
 
@@ -612,9 +616,8 @@ tuple<Status, optional<std::vector<directory_entry>>> Azure::ls_with_sizes(
     try {
       response = container_client.ListBlobsByHierarchy(delimiter, options);
     } catch (const ::Azure::Storage::StorageException& e) {
-      auto st = LOG_STATUS(Status_AzureError(
-          "List blobs failed on: " + uri_dir.to_string() + "; " + e.Message));
-      return {st, nullopt};
+      throw AzureException(
+          "List blobs failed on: " + uri_dir.to_string() + "; " + e.Message);
     }
 
     for (const auto& blob : response.Blobs) {
@@ -636,7 +639,7 @@ tuple<Status, optional<std::vector<directory_entry>>> Azure::ls_with_sizes(
     options.ContinuationToken = response.NextPageToken;
   } while (options.ContinuationToken.HasValue());
 
-  return {Status::Ok(), entries};
+  return entries;
 }
 
 Status Azure::move_object(const URI& old_uri, const URI& new_uri) {
@@ -836,7 +839,7 @@ Status Azure::remove_dir(const URI& uri) const {
   std::vector<std::string> paths;
   RETURN_NOT_OK(ls(uri, &paths, ""));
   auto status = parallel_for(thread_pool_, 0, paths.size(), [&](size_t i) {
-    RETURN_NOT_OK(remove_blob(URI(paths[i])));
+    throw_if_not_ok(remove_blob(URI(paths[i])));
     return Status::Ok();
   });
   RETURN_NOT_OK(status);
@@ -1087,6 +1090,69 @@ Status Azure::write_blocks(
   return Status::Ok();
 }
 
+LsObjects Azure::list_blobs_impl(
+    const std::string& container_name,
+    const std::string& blob_path,
+    bool recursive,
+    int max_keys,
+    optional<std::string>& continuation_token) const {
+  try {
+    ::Azure::Storage::Blobs::ListBlobsOptions options{
+        .Prefix = blob_path,
+        .ContinuationToken = to_azure_nullable(continuation_token),
+        .PageSizeHint = max_keys};
+    auto container_client = client().GetBlobContainerClient(container_name);
+    auto to_directory_entry =
+        [&container_name](const ::Azure::Storage::Blobs::Models::BlobItem& item)
+        -> LsObjects::value_type {
+      return {
+          "azure://" + container_name + "/" +
+              remove_front_slash(remove_trailing_slash(item.Name)),
+          item.BlobSize >= 0 ? static_cast<uint64_t>(item.BlobSize) : 0};
+    };
+
+    LsObjects result;
+    if (recursive) {
+      auto response = container_client.ListBlobs(options);
+
+      continuation_token = from_azure_nullable(response.NextPageToken);
+
+      result.reserve(response.Blobs.size());
+      std::transform(
+          response.Blobs.begin(),
+          response.Blobs.end(),
+          std::back_inserter(result),
+          to_directory_entry);
+    } else {
+      auto response = container_client.ListBlobsByHierarchy("/", options);
+
+      continuation_token = from_azure_nullable(response.NextPageToken);
+
+      result.reserve(response.Blobs.size() + response.BlobPrefixes.size());
+      std::transform(
+          response.Blobs.begin(),
+          response.Blobs.end(),
+          std::back_inserter(result),
+          to_directory_entry);
+      std::transform(
+          response.BlobPrefixes.begin(),
+          response.BlobPrefixes.end(),
+          std::back_inserter(result),
+          [&container_name](std::string& name) -> LsObjects::value_type {
+            return {
+                "azure://" + container_name + "/" +
+                    remove_front_slash(add_trailing_slash(name)),
+                0};
+          });
+    }
+
+    return result;
+  } catch (const ::Azure::Storage::StorageException& e) {
+    throw AzureException(
+        "List blobs failed on: " + blob_path + "; " + e.Message);
+  }
+}
+
 Status Azure::upload_block(
     const std::string& container_name,
     const std::string& blob_path,
@@ -1111,7 +1177,7 @@ Status Azure::upload_block(
 Status Azure::parse_azure_uri(
     const URI& uri,
     std::string* const container_name,
-    std::string* const blob_path) const {
+    std::string* const blob_path) {
   assert(uri.is_azure());
   const std::string uri_str = uri.to_string();
 
@@ -1189,8 +1255,7 @@ std::string Azure::BlockListUploadState::next_block_id() {
   return b64_block_id_str;
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #if defined(_WIN32)
 #include <azure/core/http/win_http_transport.hpp>
@@ -1224,9 +1289,7 @@ std::shared_ptr<::Azure::Core::Http::HttpTransport> create_transport(
   }
 
   if (!ssl_cfg.ca_path().empty()) {
-    LOG_WARN(
-        "Azure ignores the `ssl.ca_path` configuration key, "
-        "use `ssl.ca_file` instead");
+    transport_opts.CAPath = ssl_cfg.ca_path();
   }
 
   if (ssl_cfg.verify() == false) {
