@@ -95,9 +95,6 @@ class FloatScalingFilter : public Filter {
       , byte_width_(byte_width) {
   }
 
-  /** Dumps the filter details in ASCII format in the selected output. */
-  void dump(FILE* out) const override;
-
   /** Serializes this filter's metadata to the given buffer. */
   void serialize_impl(Serializer& serializer) const override;
 
@@ -106,7 +103,7 @@ class FloatScalingFilter : public Filter {
    * stores it as integers with the value round((raw_float - offset) / scale)
    * with the pre-specified byte width.
    */
-  Status run_forward(
+  void run_forward(
       const WriterTile& tile,
       WriterTile* const offsets_tile,
       FilterBuffer* input_metadata,
@@ -150,6 +147,10 @@ class FloatScalingFilter : public Filter {
    */
   Datatype output_datatype(Datatype input_type) const override;
 
+ protected:
+  /** Dumps the filter details in ASCII format in the selected output string. */
+  std::ostream& output(std::ostream& os) const override;
+
  private:
   /** The scale factor. */
   double scale_;
@@ -167,7 +168,7 @@ class FloatScalingFilter : public Filter {
    * Run forward, templated on the size of the input type.
    */
   template <typename T>
-  Status run_forward(
+  void run_forward(
       FilterBuffer* input_metadata,
       FilterBuffer* input,
       FilterBuffer* output_metadata,
@@ -177,7 +178,7 @@ class FloatScalingFilter : public Filter {
    * Run forward, templated on the size of the input type and byte width.
    */
   template <typename T, typename W>
-  Status run_forward(
+  void run_forward(
       FilterBuffer* input_metadata,
       FilterBuffer* input,
       FilterBuffer* output_metadata,
