@@ -709,6 +709,15 @@ extern const format_version_t enumerations_min_format_version;
 /** The current Enumerations version. */
 extern const format_version_t enumerations_version;
 
+/** The lowest version supported format version for CurrentDomain API. */
+extern const format_version_t current_domain_min_format_version;
+
+/** The current CurrentDomain API version. */
+extern const format_version_t current_domain_version;
+
+/** The NDRectangle current_domain */
+extern const std::string current_domain_ndrectangle_str;
+
 /** The maximum size of a tile chunk (unit of compression) in bytes. */
 extern const uint64_t max_tile_chunk_size;
 
@@ -751,6 +760,9 @@ const void* fill_value(Datatype type);
 /** The redirection header key in REST response. */
 extern const std::string redirection_header_key;
 
+/** The REST custom headers config key prefix. */
+extern const std::string rest_header_prefix;
+
 /** Delimiter for lists passed as config parameter */
 extern const std::string config_delimiter;
 
