@@ -77,6 +77,8 @@ ReaderBase::ReaderBase(
     , memory_tracker_(params.query_memory_tracker())
     , condition_(params.condition())
     , user_requested_timestamps_(false)
+    , deletes_consolidation_no_purge_(
+          buffers_.count(constants::delete_timestamps) != 0)
     , use_timestamps_(false)
     , initial_data_loaded_(false)
     , max_batch_size_(config_.get<uint64_t>("vfs.max_batch_size").value())
@@ -101,66 +103,6 @@ ReaderBase::ReaderBase(
   }
 }
 
-/* ********************************* */
-/*          STATIC FUNCTIONS         */
-/* ********************************* */
-
-template <class T>
-void ReaderBase::compute_result_space_tiles(
-    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
-    const std::vector<std::vector<uint8_t>>& tile_coords,
-    const TileDomain<T>& array_tile_domain,
-    const std::vector<TileDomain<T>>& frag_tile_domains,
-    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
-    shared_ptr<MemoryTracker> memory_tracker) {
-  auto fragment_num = (unsigned)frag_tile_domains.size();
-  auto dim_num = array_tile_domain.dim_num();
-  std::vector<T> start_coords;
-  const T* coords;
-  start_coords.resize(dim_num);
-
-  // For all tile coordinates
-  for (const auto& tc : tile_coords) {
-    coords = (T*)(&(tc[0]));
-    start_coords = array_tile_domain.start_coords(coords);
-
-    // Create result space tile and insert into the map
-    auto r =
-        result_space_tiles.emplace(coords, ResultSpaceTile<T>(memory_tracker));
-    auto& result_space_tile = r.first->second;
-    result_space_tile.set_start_coords(start_coords);
-
-    // Add fragment info to the result space tile
-    for (unsigned f = 0; f < fragment_num; ++f) {
-      // Check if the fragment overlaps with the space tile
-      if (!frag_tile_domains[f].in_tile_domain(coords))
-        continue;
-
-      // Check if any previous fragment covers this fragment
-      // for the tile identified by `coords`
-      bool covered = false;
-      for (unsigned j = 0; j < f; ++j) {
-        if (frag_tile_domains[j].covers(coords, frag_tile_domains[f])) {
-          covered = true;
-          break;
-        }
-      }
-
-      // Exclude this fragment from the space tile
-      if (covered)
-        continue;
-
-      // Include this fragment in the space tile
-      auto frag_domain = frag_tile_domains[f].domain_slice();
-      auto frag_idx = frag_tile_domains[f].id();
-      result_space_tile.append_frag_domain(frag_idx, frag_domain);
-      auto tile_idx = frag_tile_domains[f].tile_pos(coords);
-      result_space_tile.set_result_tile(
-          frag_idx, tile_idx, *fragment_metadata[frag_idx].get());
-    }
-  }
-}
-
 /* ****************************** */
 /*         PUBLIC METHODS         */
 /* ****************************** */
@@ -188,6 +130,13 @@ bool ReaderBase::skip_field(
     return true;
   }
 
+  // If an attribute exists but has changed from fixed->var or var->fixed after
+  // schema evolution, ignore for this fragment's tile offsets
+  if ((array_schema_.var_size(name) && !schema->var_size(name)) ||
+      (!array_schema_.var_size(name) && schema->var_size(name))) {
+    return true;
+  }
+
   // If the fragment doesn't include timestamps
   if (timestamps_not_present(name, frag_idx)) {
     return true;
@@ -205,6 +154,134 @@ bool ReaderBase::skip_field(
 /*        PROTECTED METHODS       */
 /* ****************************** */
 
+std::vector<uint64_t> ReaderBase::tile_offset_sizes() {
+  auto timer_se = stats_->start_timer("tile_offset_sizes");
+
+  // For easy reference.
+  std::vector<uint64_t> ret(fragment_metadata_.size());
+  const auto dim_num = array_schema_.dim_num();
+
+  // Compute the size of tile offsets per fragments.
+  const auto relevant_fragments = subarray_.relevant_fragments();
+  throw_if_not_ok(parallel_for(
+      &resources_.compute_tp(), 0, relevant_fragments.size(), [&](uint64_t i) {
+        // For easy reference.
+        auto frag_idx = relevant_fragments[i];
+        auto& fragment = fragment_metadata_[frag_idx];
+        const auto& schema = fragment->array_schema();
+        const auto tile_num = fragment->tile_num();
+        const auto dense = schema->dense();
+
+        // Compute the number of dimensions/attributes requiring offsets.
+        uint64_t num = 0;
+
+        // For fragments with version smaller than 5 we have zipped coords.
+        // Otherwise we load each dimensions independently.
+        if (!dense) {
+          if (fragment->version() < 5) {
+            num = 1;
+          } else {
+            for (unsigned d = 0; d < dim_num; ++d) {
+              // Fixed tile (offsets or fixed data).
+              num++;
+
+              // If var size, we load var offsets and var tile sizes.
+              if (is_dim_var_size_[d]) {
+                num += 2;
+              }
+            }
+          }
+        }
+
+        // Process everything loaded for query condition.
+        for (auto& name : qc_loaded_attr_names_) {
+          // Not a member of array schema, this field was added in array
+          // schema evolution, ignore for this fragment's tile offsets.
+          // Also skip dimensions.
+          if (!schema->is_field(name) || schema->is_dim(name)) {
+            continue;
+          }
+
+          // Fixed tile (offsets or fixed data).
+          num++;
+
+          // If var size, we load var offsets and var tile sizes.
+          const auto attr = schema->attribute(name);
+          num += 2 * attr->var_size();
+
+          // If nullable, we load nullable offsets.
+          num += attr->nullable();
+        }
+
+        // Process everything loaded for user requested data.
+        for (auto& it : buffers_) {
+          const auto& name = it.first;
+
+          // Skip dimensions and attributes loaded by query condition as they
+          // are processed above. Special attributes (timestamps, delete
+          // timestamps, etc.) are processed below.
+          if (array_schema_.is_dim(name) || !schema->is_field(name) ||
+              qc_loaded_attr_names_set_.count(name) != 0 ||
+              schema->is_special_attribute(name)) {
+            continue;
+          }
+
+          // Fixed tile (offsets or fixed data).
+          num++;
+
+          // If var size, we load var offsets and var tile sizes.
+          const auto attr = schema->attribute(name);
+          num += 2 * attr->var_size();
+
+          // If nullable, we load nullable offsets.
+          num += attr->nullable();
+        }
+
+        if (!dense) {
+          // Add timestamps if required.
+          if (!timestamps_not_present(constants::timestamps, frag_idx)) {
+            num++;
+          }
+
+          // Add delete metadata if required.
+          if (!delete_meta_not_present(
+                  constants::delete_timestamps, frag_idx)) {
+            num++;
+            num += deletes_consolidation_no_purge_;
+          }
+        }
+
+        // Finally set the size of the loaded data.
+
+        // The expected size of the tile offsets
+        unsigned offsets_size = num * tile_num * sizeof(uint64_t);
+
+        // Other than the offsets themselves, there is also memory used for the
+        // initialization of the vectors that hold them. This initialization
+        // takes place in LoadedFragmentMetadata::resize_offsets()
+
+        // Calculate the number of fields
+        unsigned num_fields = schema->attribute_num() + 1 +
+                              fragment->has_timestamps() +
+                              fragment->has_delete_meta() * 2;
+
+        // If version < 5 we use zipped coordinates, otherwise separate
+        num_fields += (fragment->version() >= 5) ? schema->dim_num() : 0;
+
+        // The additional memory required for the vectors to
+        // store the tile offsets. The number of fields is calculated above.
+        // Each vector requires 32 bytes. Each field requires 4 vectors. These
+        // are: tile_offsets_, tile_var_offsets_, tile_var_sizes_,
+        // tile_validity_offsets_ and are located in loaded_fragment_metadata.h
+        unsigned offsets_init_size = num_fields * 4 * 32;
+
+        ret[frag_idx] = offsets_size + offsets_init_size;
+        return Status::Ok();
+      }));
+
+  return ret;
+}
+
 bool ReaderBase::process_partial_timestamps(FragmentMetadata& frag_meta) const {
   return frag_meta.has_timestamps() &&
          frag_meta.partial_time_overlap(
@@ -432,7 +509,7 @@ void ReaderBase::load_tile_offsets(
   const auto encryption_key = array_->encryption_key();
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       relevant_fragments.size(),
       [&](const uint64_t i) {
@@ -450,7 +527,8 @@ void ReaderBase::load_tile_offsets(
           filtered_names.emplace_back(name);
         }
 
-        fragment->load_tile_offsets(*encryption_key, filtered_names);
+        fragment->loaded_metadata()->load_tile_offsets(
+            *encryption_key, filtered_names);
         return Status::Ok();
       }));
 }
@@ -462,7 +540,7 @@ void ReaderBase::load_tile_var_sizes(
   const auto encryption_key = array_->encryption_key();
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       relevant_fragments.size(),
       [&](const uint64_t i) {
@@ -482,7 +560,8 @@ void ReaderBase::load_tile_var_sizes(
             continue;
           }
 
-          fragment->load_tile_var_sizes(*encryption_key, name);
+          fragment->loaded_metadata()->load_tile_var_sizes(
+              *encryption_key, name);
         }
 
         return Status::Ok();
@@ -496,7 +575,7 @@ void ReaderBase::load_tile_metadata(
   const auto encryption_key = array_->encryption_key();
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       relevant_fragments.size(),
       [&](const uint64_t i) {
@@ -518,10 +597,14 @@ void ReaderBase::load_tile_metadata(
           }
         }
 
-        fragment->load_tile_max_values(*encryption_key, to_load);
-        fragment->load_tile_min_values(*encryption_key, to_load);
-        fragment->load_tile_sum_values(*encryption_key, to_load);
-        fragment->load_tile_null_count_values(*encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_max_values(
+            *encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_min_values(
+            *encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_sum_values(
+            *encryption_key, to_load);
+        fragment->loaded_metadata()->load_tile_null_count_values(
+            *encryption_key, to_load);
 
         return Status::Ok();
       }));
@@ -533,14 +616,15 @@ void ReaderBase::load_processed_conditions() {
 
   // Load all fragments in parallel.
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       fragment_metadata_.size(),
       [&](const uint64_t i) {
         auto& fragment = fragment_metadata_[i];
 
         if (fragment->has_delete_meta()) {
-          fragment->load_processed_conditions(*encryption_key);
+          fragment->loaded_metadata()->load_processed_conditions(
+              *encryption_key);
         }
 
         return Status::Ok();
@@ -549,7 +633,7 @@ void ReaderBase::load_processed_conditions() {
 
 Status ReaderBase::read_and_unfilter_attribute_tiles(
     const std::vector<NameToLoad>& names,
-    const std::vector<ResultTile*>& result_tiles) const {
+    const std::vector<ResultTile*>& result_tiles) {
   // The filtered data here contains the memory allocations for all of the
   // filtered data that is read by `read_attribute_tiles`. To prevent
   // modifications to the filter pipeline at the moment, the `result_tiles`
@@ -576,7 +660,7 @@ Status ReaderBase::read_and_unfilter_attribute_tiles(
 
 Status ReaderBase::read_and_unfilter_coordinate_tiles(
     const std::vector<std::string>& names,
-    const std::vector<ResultTile*>& result_tiles) const {
+    const std::vector<ResultTile*>& result_tiles) {
   // See the comment in 'read_and_unfilter_attribute_tiles' to get more
   // information about the lifetime of this object.
   auto filtered_data{read_coordinate_tiles(names, result_tiles)};
@@ -626,6 +710,7 @@ std::list<FilteredData> ReaderBase::read_tiles(
     const bool var_sized{array_schema_.var_size(name)};
     const bool nullable{array_schema_.is_nullable(name)};
     filtered_data.emplace_back(
+        resources_,
         *this,
         min_batch_size_,
         max_batch_size_,
@@ -636,7 +721,6 @@ std::list<FilteredData> ReaderBase::read_tiles(
         var_sized,
         nullable,
         val_only,
-        storage_manager_,
         read_tasks,
         memory_tracker_);
 
@@ -696,7 +780,7 @@ std::list<FilteredData> ReaderBase::read_tiles(
   stats_->add_counter("num_tiles_read", num_tiles_read);
 
   // Wait for the read tasks to finish.
-  auto statuses{storage_manager_->io_tp()->wait_all_status(read_tasks)};
+  auto statuses{resources_.io_tp().wait_all_status(read_tasks)};
   for (const auto& st : statuses) {
     throw_if_not_ok(st);
   }
@@ -820,7 +904,7 @@ Status ReaderBase::post_process_unfiltered_tile(
 Status ReaderBase::unfilter_tiles(
     const std::string& name,
     const bool validity_only,
-    const std::vector<ResultTile*>& result_tiles) const {
+    const std::vector<ResultTile*>& result_tiles) {
   const auto stat_type = (array_schema_.is_attr(name)) ? "unfilter_attr_tiles" :
                                                          "unfilter_coord_tiles";
 
@@ -842,7 +926,7 @@ Status ReaderBase::unfilter_tiles(
 
   // Compute parallelization parameters.
   uint64_t num_range_threads = 1;
-  const auto num_threads = storage_manager_->compute_tp()->concurrency_level();
+  const auto num_threads = resources_.compute_tp().concurrency_level();
   if (chunking && num_tiles < num_threads) {
     // Ceil the division between thread_num and num_tiles.
     num_range_threads = 1 + ((num_threads - 1) / num_tiles);
@@ -859,7 +943,7 @@ Status ReaderBase::unfilter_tiles(
 
   // Pre-compute chunk offsets.
   auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, num_tiles, [&, this](uint64_t i) {
+      &resources_.compute_tp(), 0, num_tiles, [&, this](uint64_t i) {
         auto&& [st, tile_size, tile_var_size, tile_validity_size] =
             load_tile_chunk_data(
                 name,
@@ -870,7 +954,7 @@ Status ReaderBase::unfilter_tiles(
                 tiles_chunk_data[i],
                 tiles_chunk_var_data[i],
                 tiles_chunk_validity_data[i]);
-        RETURN_NOT_OK(st);
+        throw_if_not_ok(st);
         unfiltered_tile_size[i] = tile_size.value();
         unfiltered_tile_var_size[i] = tile_var_size.value();
         unfiltered_tile_validity_size[i] = tile_validity_size.value();
@@ -883,13 +967,13 @@ Status ReaderBase::unfilter_tiles(
 
   // Unfilter all tiles/chunks in parallel using the precomputed offsets.
   status = parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       num_tiles,
       0,
       num_range_threads,
       [&](uint64_t i, uint64_t range_thread_idx) {
-        return unfilter_tile(
+        throw_if_not_ok(unfilter_tile(
             name,
             validity_only,
             result_tiles[i],
@@ -899,7 +983,8 @@ Status ReaderBase::unfilter_tiles(
             num_range_threads,
             tiles_chunk_data[i],
             tiles_chunk_var_data[i],
-            tiles_chunk_validity_data[i]);
+            tiles_chunk_validity_data[i]));
+        return Status::Ok();
       });
   RETURN_CANCEL_OR_ERROR(status);
 
@@ -974,7 +1059,7 @@ Status ReaderBase::unfilter_tile(
         t_var->type(), array_schema_.version());
   }
 
-  auto concurrency_level = storage_manager_->compute_tp()->concurrency_level();
+  auto concurrency_level = resources_.compute_tp().concurrency_level();
 
   if (!validity_only) {
     // Unfiltered fixed data
@@ -994,7 +1079,7 @@ Status ReaderBase::unfilter_tile(
           t_min,
           t_max,
           concurrency_level,
-          storage_manager_->config()));
+          resources_.config()));
     }
 
     // Prevent processing past the end of chunks in case there are more
@@ -1013,7 +1098,7 @@ Status ReaderBase::unfilter_tile(
           tvar_min,
           tvar_max,
           concurrency_level,
-          storage_manager_->config()));
+          resources_.config()));
     }
   }
 
@@ -1037,7 +1122,7 @@ Status ReaderBase::unfilter_tile(
         tval_min,
         tval_max,
         concurrency_level,
-        storage_manager_->config()));
+        resources_.config()));
   }
 
   return Status::Ok();
@@ -1049,16 +1134,23 @@ uint64_t ReaderBase::offsets_bytesize() const {
 }
 
 uint64_t ReaderBase::get_attribute_tile_size(
-    const std::string& name, unsigned f, uint64_t t) {
+    const std::string& name, unsigned f, uint64_t t) const {
   uint64_t tile_size = 0;
-  if (!fragment_metadata_[f]->array_schema()->is_field(name)) {
+  if (skip_field(f, name)) {
     return tile_size;
   }
 
   tile_size += fragment_metadata_[f]->tile_size(name, t);
 
+  /**
+   * Note: There is a distinct case in which a schema may evolve from
+   * fixed-sized to var-sized. In this scenario, the LoadedFragmentMetadata
+   * contains fixed tiles. The tile_var_size should be calculated iff
+   * both the current _and_ loaded attributes are var-sized.
+   */
   if (array_schema_.var_size(name)) {
-    tile_size += fragment_metadata_[f]->tile_var_size(name, t);
+    tile_size +=
+        fragment_metadata_[f]->loaded_metadata()->tile_var_size(name, t);
   }
 
   if (array_schema_.is_nullable(name)) {
@@ -1069,11 +1161,31 @@ uint64_t ReaderBase::get_attribute_tile_size(
   return tile_size;
 }
 
+uint64_t ReaderBase::get_attribute_persisted_tile_size(
+    const std::string& name, unsigned f, uint64_t t) const {
+  uint64_t tile_size = 0;
+  if (skip_field(f, name)) {
+    return tile_size;
+  }
+
+  if (array_schema_.var_size(name)) {
+    tile_size +=
+        fragment_metadata_[f]->loaded_metadata()->persisted_tile_var_size(
+            name, t);
+  }
+
+  if (array_schema_.is_nullable(name)) {
+    tile_size +=
+        fragment_metadata_[f]->loaded_metadata()->persisted_tile_validity_size(
+            name, t);
+  }
+
+  return tile_size;
+}
+
 template <class T>
-void ReaderBase::compute_result_space_tiles(
-    const Subarray& subarray,
-    const Subarray& partitioner_subarray,
-    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles) const {
+std::pair<TileDomain<T>, std::vector<TileDomain<T>>>
+ReaderBase::compute_tile_domains(const Subarray& partitioner_subarray) const {
   // For easy reference
   auto domain = array_schema_.domain().domain();
   auto tile_extents = array_schema_.domain().tile_extents();
@@ -1094,19 +1206,9 @@ void ReaderBase::compute_result_space_tiles(
     }
   }
 
-  // Get tile coords and array domain
-  const auto& tile_coords = subarray.tile_coords();
-  TileDomain<T> array_tile_domain(
-      UINT32_MAX, domain, domain, tile_extents, tile_order);
-
-  // Compute result space tiles
-  compute_result_space_tiles<T>(
-      fragment_metadata_,
-      tile_coords,
-      array_tile_domain,
-      frag_tile_domains,
-      result_space_tiles,
-      query_memory_tracker_);
+  return {
+      TileDomain<T>(UINT32_MAX, domain, domain, tile_extents, tile_order),
+      frag_tile_domains};
 }
 
 bool ReaderBase::has_coords() const {
@@ -1130,10 +1232,7 @@ ReaderBase::cache_dimension_label_data() {
   std::vector<const void*> non_empty_domains(fragment_metadata_.size());
   std::vector<uint64_t> frag_first_array_tile_idx(fragment_metadata_.size());
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      fragment_metadata_.size(),
-      [&](unsigned f) {
+      &resources_.compute_tp(), 0, fragment_metadata_.size(), [&](unsigned f) {
         non_empty_domains[f] =
             fragment_metadata_[f]->non_empty_domain()[0].data();
         auto ned = static_cast<const IndexType*>(non_empty_domains[f]);
@@ -1180,10 +1279,7 @@ void ReaderBase::validate_attribute_order(
   AttributeOrderValidator validator(
       attribute_name, fragment_metadata_.size(), query_memory_tracker_);
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      fragment_metadata_.size(),
-      [&](uint64_t f) {
+      &resources_.compute_tp(), 0, fragment_metadata_.size(), [&](uint64_t f) {
         validator.find_fragments_to_check(
             array_min_idx, array_max_idx, f, non_empty_domains);
 
@@ -1191,10 +1287,7 @@ void ReaderBase::validate_attribute_order(
       }));
 
   throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(),
-      0,
-      fragment_metadata_.size(),
-      [&](int64_t f) {
+      &resources_.compute_tp(), 0, fragment_metadata_.size(), [&](int64_t f) {
         validator.validate_without_loading_tiles<IndexType, AttributeType>(
             index_dim,
             increasing_data,
@@ -1215,7 +1308,7 @@ void ReaderBase::validate_attribute_order(
 
     // Validate bounds not validated using tile data.
     throw_if_not_ok(parallel_for(
-        storage_manager_->compute_tp(),
+        &resources_.compute_tp(),
         0,
         fragment_metadata_.size(),
         [&](unsigned f) {
@@ -1264,38 +1357,22 @@ void ReaderBase::validate_attribute_order(
 }
 
 // Explicit template instantiations
-template void ReaderBase::compute_result_space_tiles<int8_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const int8_t*, ResultSpaceTile<int8_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<uint8_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const uint8_t*, ResultSpaceTile<uint8_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<int16_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const int16_t*, ResultSpaceTile<int16_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<uint16_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const uint16_t*, ResultSpaceTile<uint16_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<int32_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const int32_t*, ResultSpaceTile<int32_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<uint32_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const uint32_t*, ResultSpaceTile<uint32_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<int64_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const int64_t*, ResultSpaceTile<int64_t>>&) const;
-template void ReaderBase::compute_result_space_tiles<uint64_t>(
-    const Subarray&,
-    const Subarray&,
-    std::map<const uint64_t*, ResultSpaceTile<uint64_t>>&) const;
+template std::pair<TileDomain<int8_t>, std::vector<TileDomain<int8_t>>>
+ReaderBase::compute_tile_domains<int8_t>(const Subarray&) const;
+template std::pair<TileDomain<uint8_t>, std::vector<TileDomain<uint8_t>>>
+ReaderBase::compute_tile_domains<uint8_t>(const Subarray&) const;
+template std::pair<TileDomain<int16_t>, std::vector<TileDomain<int16_t>>>
+ReaderBase::compute_tile_domains<int16_t>(const Subarray&) const;
+template std::pair<TileDomain<uint16_t>, std::vector<TileDomain<uint16_t>>>
+ReaderBase::compute_tile_domains<uint16_t>(const Subarray&) const;
+template std::pair<TileDomain<int32_t>, std::vector<TileDomain<int32_t>>>
+ReaderBase::compute_tile_domains<int32_t>(const Subarray&) const;
+template std::pair<TileDomain<uint32_t>, std::vector<TileDomain<uint32_t>>>
+ReaderBase::compute_tile_domains<uint32_t>(const Subarray&) const;
+template std::pair<TileDomain<int64_t>, std::vector<TileDomain<int64_t>>>
+ReaderBase::compute_tile_domains<int64_t>(const Subarray&) const;
+template std::pair<TileDomain<uint64_t>, std::vector<TileDomain<uint64_t>>>
+ReaderBase::compute_tile_domains<uint64_t>(const Subarray&) const;
 template tuple<Range, std::vector<const void*>, std::vector<uint64_t>>
 ReaderBase::cache_dimension_label_data<int8_t>();
 template tuple<Range, std::vector<const void*>, std::vector<uint64_t>>
