@@ -47,7 +47,6 @@
 #include "tiledb/sm/query/readers/result_cell_slab.h"
 #include "tiledb/sm/query/readers/result_space_tile.h"
 #include "tiledb/sm/query/writers/domain_buffer.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/subarray_partitioner.h"
 
 namespace tiledb::sm {
@@ -174,35 +173,6 @@ class ReaderBase : public StrategyBase {
   /*          STATIC FUNCTIONS         */
   /* ********************************* */
 
-  /**
-   * Computes a mapping (tile coordinates) -> (result space tile).
-   * The produced result space tiles will contain information only
-   * about fragments that will contribute results. Specifically, if
-   * a fragment is completely covered by a more recent fragment
-   * in a particular space tile, then it will certainly not contribute
-   * results and, thus, no information about that fragment is included
-   * in the space tile.
-   *
-   * @tparam T The datatype of the tile domains.
-   * @param domain The array domain
-   * @param tile_coords The unique coordinates of the tiles that intersect
-   *     a subarray.
-   * @param array_tile_domain The array tile domain.
-   * @param frag_tile_domains The tile domains of each fragment. These
-   *     are assumed to be ordered from the most recent to the oldest
-   *     fragment.
-   * @param result_space_tiles The result space tiles to be produced
-   *     by the function.
-   */
-  template <class T>
-  static void compute_result_space_tiles(
-      const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
-      const std::vector<std::vector<uint8_t>>& tile_coords,
-      const TileDomain<T>& array_tile_domain,
-      const std::vector<TileDomain<T>>& frag_tile_domains,
-      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
-      shared_ptr<MemoryTracker> memory_tracker);
-
   /**
    * Computes the minimum and maximum indexes of tile chunks to process based on
    * the available threads.
@@ -299,6 +269,9 @@ class ReaderBase : public StrategyBase {
   /** If the user requested timestamps attribute in the query */
   bool user_requested_timestamps_;
 
+  /** Are we doing deletes consolidation (without purge option). */
+  bool deletes_consolidation_no_purge_;
+
   /**
    * If the special timestamps attribute should be loaded to memory for
    * this query
@@ -311,6 +284,12 @@ class ReaderBase : public StrategyBase {
    */
   std::vector<bool> timestamps_needed_for_deletes_and_updates_;
 
+  /** Are dimensions var sized. */
+  std::vector<bool> is_dim_var_size_;
+
+  /** Names of dim/attr loaded for query condition. */
+  std::vector<std::string> qc_loaded_attr_names_;
+
   /** Names of dim/attr loaded for query condition. */
   std::unordered_set<std::string> qc_loaded_attr_names_set_;
 
@@ -335,6 +314,9 @@ class ReaderBase : public StrategyBase {
    * */
   std::unordered_map<std::string, QueryBuffer>& aggregate_buffers_;
 
+  /** Per fragment tile offsets memory usage. */
+  std::vector<uint64_t> per_frag_tile_offsets_usage_;
+
   /* ********************************* */
   /*         PROTECTED METHODS         */
   /* ********************************* */
@@ -368,6 +350,13 @@ class ReaderBase : public StrategyBase {
     return true;
   }
 
+  /**
+   * Computes the required size for loading tile offsets, per fragments.
+   *
+   * @return Required memory for loading tile offsets, per fragments.
+   */
+  std::vector<uint64_t> tile_offset_sizes();
+
   /**
    * Returns if we need to process partial timestamp condition for this
    * fragment.
@@ -538,7 +527,7 @@ class ReaderBase : public StrategyBase {
    */
   Status read_and_unfilter_attribute_tiles(
       const std::vector<NameToLoad>& names,
-      const std::vector<ResultTile*>& result_tiles) const;
+      const std::vector<ResultTile*>& result_tiles);
 
   /**
    * Read and unfilter coordinate tiles.
@@ -550,7 +539,7 @@ class ReaderBase : public StrategyBase {
    */
   Status read_and_unfilter_coordinate_tiles(
       const std::vector<std::string>& names,
-      const std::vector<ResultTile*>& result_tiles) const;
+      const std::vector<ResultTile*>& result_tiles);
 
   /**
    * Concurrently executes across each name in `names` and each result tile
@@ -613,7 +602,7 @@ class ReaderBase : public StrategyBase {
   Status unfilter_tiles(
       const std::string& name,
       const bool validity_only,
-      const std::vector<ResultTile*>& result_tiles) const;
+      const std::vector<ResultTile*>& result_tiles);
 
   /**
    * Unfilter a specific range of chunks in tile
@@ -659,21 +648,29 @@ class ReaderBase : public StrategyBase {
    * @return Tile size.
    */
   uint64_t get_attribute_tile_size(
-      const std::string& name, unsigned f, uint64_t t);
+      const std::string& name, unsigned f, uint64_t t) const;
 
   /**
-   * Computes the result space tiles based on the current partition.
+   * Get the on disk size of an attribute tile.
+   *
+   * @param name The attribute name.
+   * @param f The fragment idx.
+   * @param t The tile idx.
+   * @return Tile size.
+   */
+  uint64_t get_attribute_persisted_tile_size(
+      const std::string& name, unsigned f, uint64_t t) const;
+
+  /**
+   * Computes the tile domains based on the current partition.
    *
    * @tparam T The domain datatype.
-   * @param subarray The input subarray.
    * @param partitioner_subarray The partitioner subarray.
-   * @param result_space_tiles The result space tiles to be computed.
+   * @return array tile domain, fragments tile domains.
    */
   template <class T>
-  void compute_result_space_tiles(
-      const Subarray& subarray,
-      const Subarray& partitioner_subarray,
-      std::map<const T*, ResultSpaceTile<T>>& result_space_tiles) const;
+  std::pair<TileDomain<T>, std::vector<TileDomain<T>>> compute_tile_domains(
+      const Subarray& partitioner_subarray) const;
 
   /** Returns `true` if the coordinates are included in the attributes. */
   bool has_coords() const;
