@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,24 +39,20 @@
 #include "tiledb/sm/misc/comparators.h"
 #include "tiledb/sm/misc/hilbert.h"
 #include "tiledb/sm/misc/parallel_functions.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/query/readers/result_tile.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/subarray/subarray.h"
 
 using namespace tiledb;
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
-class SparseUnorderedWithDupsReaderStatusException : public StatusException {
+class SparseUnorderedWithDupsReaderException : public StatusException {
  public:
-  explicit SparseUnorderedWithDupsReaderStatusException(
-      const std::string& message)
+  explicit SparseUnorderedWithDupsReaderException(const std::string& message)
       : StatusException("SparseUnorderedWithDupsReader", message) {
   }
 };
@@ -84,7 +80,7 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
                &partial_tile_offsets_loading_,
                &found)
            .ok()) {
-    throw SparseUnorderedWithDupsReaderStatusException("Cannot get setting");
+    throw SparseUnorderedWithDupsReaderException("Cannot get setting");
   }
   assert(found);
 }
@@ -95,7 +91,7 @@ SparseUnorderedWithDupsReader<BitmapType>::SparseUnorderedWithDupsReader(
 
 template <class BitmapType>
 bool SparseUnorderedWithDupsReader<BitmapType>::incomplete() const {
-  return !read_state_.done_adding_result_tiles_ ||
+  return !read_state_.done_adding_result_tiles() ||
          !result_tiles_leftover_.empty();
 }
 
@@ -122,9 +118,6 @@ void SparseUnorderedWithDupsReader<BitmapType>::refresh_config() {
 
 template <class BitmapType>
 Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
-  // Subarray is not known to be explicitly set until buffers are deserialized
-  include_coords_ = subarray_.is_set();
-
   auto timer_se = stats_->start_timer("dowork");
   stats_->add_counter("loop_num", 1);
 
@@ -149,10 +142,14 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
 
   // Handle empty array.
   if (fragment_metadata_.empty()) {
-    read_state_.done_adding_result_tiles_ = true;
+    read_state_.set_done_adding_result_tiles(true);
     return Status::Ok();
   }
 
+  // Subarray is not known to be explicitly set until buffers are deserialized
+  subarray_.reset_default_ranges();
+  include_coords_ = subarray_.is_set();
+
   // Load initial data, if not loaded already. Coords are only included if the
   // subarray is set.
   throw_if_not_ok(load_initial_data());
@@ -176,7 +173,7 @@ Status SparseUnorderedWithDupsReader<BitmapType>::dowork() {
 
       // No more tiles to process, done.
       if (result_tiles.empty()) {
-        assert(read_state_.done_adding_result_tiles_);
+        assert(read_state_.done_adding_result_tiles());
         break;
       }
 
@@ -251,9 +248,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::load_tile_offsets_data() {
   bool initial_load =
       tile_offsets_min_frag_idx_ == std::numeric_limits<unsigned>::max() &&
       tile_offsets_max_frag_idx_ == 0;
-  uint64_t available_memory = array_memory_tracker_->get_memory_available() -
-                              array_memory_tracker_->get_memory_usage(
-                                  MemoryTracker::MemoryType::TILE_OFFSETS);
+  uint64_t available_memory = array_memory_tracker_->get_memory_available();
   auto& relevant_fragments = subarray_.relevant_fragments();
 
   if (!partial_tile_offsets_loading_) {
@@ -264,12 +259,12 @@ void SparseUnorderedWithDupsReader<BitmapType>::load_tile_offsets_data() {
       // tile offsets data.
       uint64_t total_tile_offset_usage = tile_offsets_size(relevant_fragments);
       if (total_tile_offset_usage > available_memory) {
-        throw SparseUnorderedWithDupsReaderStatusException(
+        throw SparseUnorderedWithDupsReaderException(
             "Cannot load tile offsets, computed size (" +
             std::to_string(total_tile_offset_usage) +
             ") is larger than available memory (" +
             std::to_string(available_memory) +
-            "). Total budget for array data (" +
+            "), increase memory budget. Total budget for array data (" +
             std::to_string(array_memory_tracker_->get_memory_budget()) + ").");
       }
 
@@ -291,7 +286,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::load_tile_offsets_data() {
         for (unsigned f = tile_offsets_min_frag_idx_;
              f < tile_offsets_max_frag_idx_;
              f++) {
-          fragment_metadata_[f]->free_tile_offsets();
+          fragment_metadata_[f]->loaded_metadata()->free_tile_offsets();
         }
 
         tile_offsets_min_frag_idx_ = tile_offsets_max_frag_idx_;
@@ -314,7 +309,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::load_tile_offsets_data() {
 
       // Make sure plan to load tile offsets for at least one fragment.
       if (tile_offsets_min_frag_idx_ == tile_offsets_max_frag_idx_) {
-        throw SparseUnorderedWithDupsReaderStatusException(
+        throw SparseUnorderedWithDupsReaderException(
             "Cannot load tile offsets for only one fragment. Offsets size for "
             "the fragment (" +
             std::to_string(
@@ -372,7 +367,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
   // memory limit as the upper memory limit, whichever is smaller.
   const uint64_t upper_memory_limit = std::min<uint64_t>(
       memory_budget_.tile_upper_memory_limit(),
-      memory_budget_.total_budget() * memory_budget_.ratio_coords());
+      memory_budget_.coordinates_budget());
 
   // Calculate memory consumption for this tile.
   auto tiles_size = get_coord_tiles_size(dim_num, f, t);
@@ -383,15 +378,22 @@ bool SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
     // available memory, allow loading it so we can make progress.
     if (!result_tiles.empty() || tiles_size > available_memory()) {
       logger_->debug(
-          "Budget exceeded adding result tiles, fragment {0}, tile "
-          "{1}",
+          "Total memory budget of {0} exceeded adding result tiles with upper "
+          "memory limit of {1}, fragment "
+          "{2}, tile {3}/{4} with size {5}",
+          memory_budget_.total_budget(),
+          upper_memory_limit,
           f,
-          t);
+          t,
+          last_t,
+          tiles_size);
 
       // Make sure we can add at least one tile.
       if (result_tiles.empty()) {
-        throw SparseUnorderedWithDupsReaderStatusException(
-            "Cannot load a single tile, increase memory budget");
+        throw SparseUnorderedWithDupsReaderException(
+            "Cannot load a single tile requiring " +
+            std::to_string(tiles_size) + " bytes, increase memory budget (" +
+            std::to_string(memory_budget_.total_budget()) + ")");
       }
 
       return true;
@@ -402,7 +404,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::add_result_tile(
   memory_used_for_coords_total_ += tiles_size;
 
   // Add the result tile.
-  result_tiles.emplace_back(f, t, frag_md);
+  result_tiles.emplace_back(f, t, frag_md, query_memory_tracker_);
 
   // Are all tiles loaded for this fragment.
   if (t == last_t) {
@@ -468,7 +470,7 @@ SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
         auto tile_num = fragment_metadata_[f]->tile_num();
 
         // Figure out the start index.
-        auto start = read_state_.frag_idx_[f].tile_idx_;
+        auto start = read_state_.frag_idx()[f].tile_idx_;
 
         // Add all tiles for this fragment.
         if (start == tile_num) {
@@ -502,7 +504,7 @@ SparseUnorderedWithDupsReader<BitmapType>::create_result_tiles() {
     logger_->debug("All result tiles loaded");
   }
 
-  read_state_.done_adding_result_tiles_ = done_adding_result_tiles;
+  read_state_.set_done_adding_result_tiles(done_adding_result_tiles);
 
   return result_tiles;
 }
@@ -764,7 +766,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_offsets_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_tiles.size(),
       0,
@@ -776,7 +778,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_offsets_tiles(
         // We might have a partially processed result tile from last run.
         auto min_pos_tile = 0;
         if (i == 0) {
-          min_pos_tile = read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
+          min_pos_tile = read_state_.frag_idx()[rt->frag_idx()].cell_idx_;
         }
 
         auto max_pos_tile =
@@ -870,7 +872,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_var_data_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_tiles.size(),
       0,
@@ -1242,7 +1244,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_fixed_data_tiles(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_tiles.size(),
       0,
@@ -1254,7 +1256,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::copy_fixed_data_tiles(
         // We might have a partially processed result tile from last run.
         auto min_pos_tile = 0;
         if (i == 0) {
-          min_pos_tile = read_state_.frag_idx_[rt->frag_idx()].cell_idx_;
+          min_pos_tile = read_state_.frag_idx()[rt->frag_idx()].cell_idx_;
         }
 
         auto max_pos_tile =
@@ -1424,7 +1426,7 @@ SparseUnorderedWithDupsReader<BitmapType>::resize_fixed_results_to_copy(
 
   uint64_t initial_cell_offset = cells_copied(names);
   uint64_t first_tile_min_pos =
-      read_state_.frag_idx_[result_tiles[0]->frag_idx()].cell_idx_;
+      read_state_.frag_idx()[result_tiles[0]->frag_idx()].cell_idx_;
 
   return resize_fixed_result_tiles_to_copy(
       max_num_cells, initial_cell_offset, first_tile_min_pos, result_tiles);
@@ -1445,8 +1447,8 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
   uint64_t max_rt_idx = result_tiles.size();
   std::mutex max_rt_idx_mtx;
   std::vector<uint64_t> total_mem_usage_per_attr(names.size());
-  throw_if_not_ok(parallel_for(
-      storage_manager_->compute_tp(), 0, names.size(), [&](uint64_t i) {
+  throw_if_not_ok(
+      parallel_for(&resources_.compute_tp(), 0, names.size(), [&](uint64_t i) {
         // For easy reference.
         const auto& name = names[i];
         const bool agg_only = aggregate_only(name);
@@ -1514,7 +1516,7 @@ SparseUnorderedWithDupsReader<BitmapType>::respect_copy_memory_budget(
       }));
 
   if (max_rt_idx == 0) {
-    throw SparseUnorderedWithDupsReaderStatusException(
+    throw SparseUnorderedWithDupsReaderException(
         "Unable to copy one tile with current budget/buffers");
   }
 
@@ -1624,7 +1626,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
 
   // Compute parallelization parameters.
   uint64_t num_range_threads = 1;
-  const auto num_threads = storage_manager_->compute_tp()->concurrency_level();
+  const auto num_threads = resources_.compute_tp().concurrency_level();
   if (result_tiles.size() < num_threads) {
     // Ceil the division between thread_num and tile_num.
     num_range_threads = 1 + ((num_threads - 1) / result_tiles.size());
@@ -1698,7 +1700,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
       if (aggregates_.count(name) != 0) {
         for (auto& aggregates : aggregates_[name]) {
           if (aggregates->need_recompute_on_overflow()) {
-            throw SparseUnorderedWithDupsReaderStatusException(
+            throw SparseUnorderedWithDupsReaderException(
                 "Overflow happened after aggregate was computed, aggregate "
                 "recompute pass is not yet implemented");
           }
@@ -1713,7 +1715,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
   if (result_tiles.size() > 0) {
     auto last_tile =
         (UnorderedWithDupsResultTile<BitmapType>*)result_tiles.back();
-    auto& frag_tile_idx = read_state_.frag_idx_[last_tile->frag_idx()];
+    auto& frag_tile_idx = read_state_.frag_idx()[last_tile->frag_idx()];
     last_tile_cells_copied = cell_offsets[result_tiles.size()] -
                              cell_offsets[result_tiles.size() - 1];
     if (frag_tile_idx.tile_idx_ == last_tile->tile_idx()) {
@@ -1724,18 +1726,20 @@ bool SparseUnorderedWithDupsReader<BitmapType>::process_tiles(
 
   // Adjust tile index.
   for (auto rt : result_tiles) {
-    read_state_.frag_idx_[rt->frag_idx()] = FragIdx(rt->tile_idx() + 1, 0);
+    read_state_.set_frag_idx(rt->frag_idx(), FragIdx(rt->tile_idx() + 1, 0));
   }
 
   // If the last tile is not fully copied, save the cell index.
   if (result_tiles.size() > 0) {
     auto last_tile =
         (UnorderedWithDupsResultTile<BitmapType>*)result_tiles.back();
-    auto& frag_tile_idx = read_state_.frag_idx_[last_tile->frag_idx()];
     if (last_tile->result_num() != last_tile_cells_copied) {
-      frag_tile_idx.tile_idx_ = last_tile->tile_idx();
-      frag_tile_idx.cell_idx_ =
-          last_tile->pos_with_given_result_sum(0, last_tile_cells_copied) + 1;
+      read_state_.set_frag_idx(
+          last_tile->frag_idx(),
+          FragIdx(
+              last_tile->tile_idx(),
+              last_tile->pos_with_given_result_sum(0, last_tile_cells_copied) +
+                  1));
     }
   }
 
@@ -1803,7 +1807,7 @@ bool SparseUnorderedWithDupsReader<BitmapType>::copy_tiles(
   uint64_t var_buffer_size = 0;
   if (var_sized) {
     uint64_t first_tile_min_pos =
-        read_state_.frag_idx_[result_tiles[0]->frag_idx()].cell_idx_;
+        read_state_.frag_idx()[result_tiles[0]->frag_idx()].cell_idx_;
 
     // Adjust the offsets buffer and make sure all data fits.
     auto&& [caused_overflow, new_var_buffer_size, new_result_tiles_size] =
@@ -1923,7 +1927,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::process_aggregates(
 
   // Process all tiles/cells in parallel.
   throw_if_not_ok(parallel_for_2d(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       result_tiles.size(),
       0,
@@ -1934,7 +1938,7 @@ void SparseUnorderedWithDupsReader<BitmapType>::process_aggregates(
 
         // The first tile might have already been processed by the last
         // computation. We only process a tile the first time.
-        if (i == 0 && read_state_.frag_idx_[rt->frag_idx()].cell_idx_ != 0) {
+        if (i == 0 && read_state_.frag_idx()[rt->frag_idx()].cell_idx_ != 0) {
           return Status::Ok();
         }
 
@@ -2001,9 +2005,10 @@ template <class BitmapType>
 void SparseUnorderedWithDupsReader<BitmapType>::end_iteration(
     ResultTilesList& result_tiles) {
   // Clear result tiles that are not necessary anymore.
-  while (!result_tiles.empty() &&
-         result_tiles.front().tile_idx() <
-             read_state_.frag_idx_[result_tiles.front().frag_idx()].tile_idx_) {
+  while (
+      !result_tiles.empty() &&
+      result_tiles.front().tile_idx() <
+          read_state_.frag_idx()[result_tiles.front().frag_idx()].tile_idx_) {
     const auto f = result_tiles.front().frag_idx();
 
     remove_result_tile(f, result_tiles, result_tiles.begin());
@@ -2030,5 +2035,4 @@ template SparseUnorderedWithDupsReader<uint8_t>::SparseUnorderedWithDupsReader(
 template SparseUnorderedWithDupsReader<uint64_t>::SparseUnorderedWithDupsReader(
     stats::Stats*, shared_ptr<Logger>, StrategyParams&);
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
