@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,11 +44,9 @@
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_math.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/hilbert_order.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/sm/tile/tile_metadata_generator.h"
 #include "tiledb/sm/tile/writer_tile_tuple.h"
@@ -60,9 +58,9 @@ using namespace tiledb::sm::stats;
 
 namespace tiledb::sm {
 
-class GlobalOrderWriterStatusException : public StatusException {
+class GlobalOrderWriterException : public StatusException {
  public:
-  explicit GlobalOrderWriterStatusException(const std::string& message)
+  explicit GlobalOrderWriterException(const std::string& message)
       : StatusException("GlobalOrderWriter", message) {
   }
 };
@@ -96,14 +94,14 @@ GlobalOrderWriter::GlobalOrderWriter(
     , current_fragment_size_(0) {
   // Check the layout is global order.
   if (layout_ != Layout::GLOBAL_ORDER) {
-    throw GlobalOrderWriterStatusException(
+    throw GlobalOrderWriterException(
         "Failed to initialize global order writer. Layout " +
         layout_str(layout_) + " is not global order.");
   }
 
   // Check no ordered attributes.
   if (array_schema_.has_ordered_attributes()) {
-    throw GlobalOrderWriterStatusException(
+    throw GlobalOrderWriterException(
         "Failed to initialize global order writer. Global order writes to "
         "ordered attributes are not yet supported.");
   }
@@ -186,8 +184,7 @@ Status GlobalOrderWriter::alloc_global_write_state() {
   // Alloc FragmentMetadata object
   global_write_state_->frag_meta_ = this->create_fragment_metadata();
   // Used in serialization when FragmentMetadata is built from ground up
-  global_write_state_->frag_meta_->set_context_resources(
-      &storage_manager_->resources());
+  global_write_state_->frag_meta_->set_context_resources(&resources_);
 
   return Status::Ok();
 }
@@ -246,7 +243,7 @@ GlobalOrderWriter::multipart_upload_state(bool client) {
   for (const auto& name : buffer_names()) {
     auto uri = meta->uri(name);
 
-    auto&& [st2, state] = storage_manager_->vfs()->multipart_upload_state(uri);
+    auto&& [st2, state] = resources_.vfs().multipart_upload_state(uri);
     RETURN_NOT_OK_TUPLE(st2, {});
     // If there is no entry for this uri, probably multipart upload is disabled
     // or no write was issued so far
@@ -257,8 +254,7 @@ GlobalOrderWriter::multipart_upload_state(bool client) {
 
     if (array_schema_.var_size(name)) {
       auto var_uri = meta->var_uri(name);
-      auto&& [st, var_state] =
-          storage_manager_->vfs()->multipart_upload_state(var_uri);
+      auto&& [st, var_state] = resources_.vfs().multipart_upload_state(var_uri);
       RETURN_NOT_OK_TUPLE(st, {});
       result[var_uri.remove_trailing_slash().last_path_part()] =
           std::move(*var_state);
@@ -266,7 +262,7 @@ GlobalOrderWriter::multipart_upload_state(bool client) {
     if (array_schema_.is_nullable(name)) {
       auto validity_uri = meta->validity_uri(name);
       auto&& [st, val_state] =
-          storage_manager_->vfs()->multipart_upload_state(validity_uri);
+          resources_.vfs().multipart_upload_state(validity_uri);
       RETURN_NOT_OK_TUPLE(st, {});
       result[validity_uri.remove_trailing_slash().last_path_part()] =
           std::move(*val_state);
@@ -288,8 +284,7 @@ Status GlobalOrderWriter::set_multipart_upload_state(
   // uri in this case holds only the buffer name
   auto absolute_uri =
       global_write_state_->frag_meta_->fragment_uri().join_path(uri);
-  return storage_manager_->vfs()->set_multipart_upload_state(
-      absolute_uri, state);
+  return resources_.vfs().set_multipart_upload_state(absolute_uri, state);
 }
 
 /* ****************************** */
@@ -328,10 +323,7 @@ Status GlobalOrderWriter::check_coord_dups() const {
   }
 
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -428,7 +420,7 @@ Status GlobalOrderWriter::check_global_order() const {
 
   // Check if all coordinates are in global order in parallel
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       coords_info_.coords_num_ - 1,
       [&](uint64_t i) {
@@ -479,7 +471,7 @@ Status GlobalOrderWriter::check_global_order_hilbert() const {
 
   // Check if all coordinates are in hilbert order in parallel
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       coords_info_.coords_num_ - 1,
       [&](uint64_t i) {
@@ -509,13 +501,13 @@ void GlobalOrderWriter::clean_up() {
     // Cleanup the fragment we are currently writing. There is a chance that the
     // URI is empty if creating the first fragment had failed.
     if (!uri.empty()) {
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(uri));
     }
     global_write_state_.reset(nullptr);
 
     // Cleanup all fragments pending commit.
     for (auto& uri : frag_uris_to_commit_) {
-      throw_if_not_ok(storage_manager_->vfs()->remove_dir(uri));
+      throw_if_not_ok(resources_.vfs().remove_dir(uri));
     }
     frag_uris_to_commit_.clear();
   }
@@ -577,10 +569,7 @@ Status GlobalOrderWriter::compute_coord_dups(
 
   std::mutex mtx;
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -675,7 +664,7 @@ Status GlobalOrderWriter::finalize_global_write_state() {
     // values.
     if (disable_checks_consolidation_) {
       auto expanded_subarray = subarray_.ndrange(0);
-      domain.expand_to_tiles(&expanded_subarray);
+      array_schema_.current_domain().expand_to_tiles(domain, expanded_subarray);
       expected_cell_num = domain.cell_num(expanded_subarray);
     }
 
@@ -709,7 +698,7 @@ Status GlobalOrderWriter::finalize_global_write_state() {
   // Write either one commit file or a consolidated commit file if multiple
   // fragments were written.
   if (frag_uris_to_commit_.size() == 0) {
-    RETURN_NOT_OK(storage_manager_->vfs()->touch(commit_uri));
+    throw_if_not_ok(resources_.vfs().touch(commit_uri));
   } else {
     std::vector<URI> commit_uris;
     commit_uris.reserve(frag_uris_to_commit_.size() + 1);
@@ -720,10 +709,7 @@ Status GlobalOrderWriter::finalize_global_write_state() {
 
     auto write_version = array_->array_schema_latest().write_version();
     Consolidator::write_consolidated_commits_file(
-        write_version,
-        array_->array_directory(),
-        commit_uris,
-        storage_manager_);
+        write_version, array_->array_directory(), commit_uris, resources_);
   }
 
   // Delete global write state
@@ -808,7 +794,7 @@ Status GlobalOrderWriter::global_write() {
     frag_meta->set_num_tiles(new_num_tiles);
 
     if (new_num_tiles == 0) {
-      throw GlobalOrderWriterStatusException(
+      throw GlobalOrderWriterException(
           "Fragment size is too small to write a single tile");
     }
 
@@ -861,7 +847,7 @@ Status GlobalOrderWriter::global_write_handle_last_tile() {
 void GlobalOrderWriter::nuke_global_write_state() {
   auto meta = global_write_state_->frag_meta_;
   throw_if_not_ok(close_files(meta));
-  throw_if_not_ok(storage_manager_->vfs()->remove_dir(meta->fragment_uri()));
+  throw_if_not_ok(resources_.vfs().remove_dir(meta->fragment_uri()));
   global_write_state_.reset(nullptr);
 }
 
@@ -879,15 +865,14 @@ Status GlobalOrderWriter::prepare_full_tiles(
   }
 
   auto num = buffers_.size();
-  auto status =
-      parallel_for(storage_manager_->compute_tp(), 0, num, [&](uint64_t i) {
-        auto buff_it = buffers_.begin();
-        std::advance(buff_it, i);
-        const auto& name = buff_it->first;
-        RETURN_CANCEL_OR_ERROR(
-            prepare_full_tiles(name, coord_dups, &tiles->at(name)));
-        return Status::Ok();
-      });
+  auto status = parallel_for(&resources_.compute_tp(), 0, num, [&](uint64_t i) {
+    auto buff_it = buffers_.begin();
+    std::advance(buff_it, i);
+    const auto& name = buff_it->first;
+    throw_if_not_ok(prepare_full_tiles(name, coord_dups, &tiles->at(name)));
+    throw_if_cancelled();
+    return Status::Ok();
+  });
 
   RETURN_NOT_OK(status);
 
