@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,11 +44,9 @@
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/tdb_math.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/hilbert_order.h"
 #include "tiledb/sm/query/query_macros.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/sm/tile/tile_metadata_generator.h"
 #include "tiledb/sm/tile/writer_tile_tuple.h"
@@ -170,7 +168,7 @@ Status UnorderedWriter::alloc_frag_meta() {
   // Alloc FragmentMetadata object.
   frag_meta_ = this->create_fragment_metadata();
   // Used in serialization when FragmentMetadata is built from ground up.
-  frag_meta_->set_context_resources(&storage_manager_->resources());
+  frag_meta_->set_context_resources(&resources_);
 
   return Status::Ok();
 }
@@ -181,7 +179,7 @@ Status UnorderedWriter::alloc_frag_meta() {
 
 void UnorderedWriter::clean_up() {
   if (frag_uri_.has_value()) {
-    throw_if_not_ok(storage_manager_->vfs()->remove_dir(frag_uri_.value()));
+    throw_if_not_ok(resources_.vfs().remove_dir(frag_uri_.value()));
   }
 }
 
@@ -217,10 +215,7 @@ Status UnorderedWriter::check_coord_dups() const {
   }
 
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -276,7 +271,7 @@ Status UnorderedWriter::check_coord_dups() const {
         return Status::Ok();
       });
 
-  RETURN_NOT_OK_ELSE(status, logger_->status_no_return_value(status));
+  RETURN_NOT_OK_ELSE(status, logger_->error(status.message()));
 
   return Status::Ok();
 }
@@ -310,10 +305,7 @@ Status UnorderedWriter::compute_coord_dups() {
 
   std::mutex mtx;
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
-      1,
-      coords_info_.coords_num_,
-      [&](uint64_t i) {
+      &resources_.compute_tp(), 1, coords_info_.coords_num_, [&](uint64_t i) {
         // Check for duplicate in adjacent cells
         bool found_dup = true;
         for (unsigned d = 0; d < dim_num; ++d) {
@@ -389,12 +381,12 @@ Status UnorderedWriter::prepare_tiles(
   }
 
   // Prepare tiles for all attributes and coordinates
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, tiles->size(), [&](uint64_t i) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, tiles->size(), [&](uint64_t i) {
         auto tiles_it = tiles->begin();
         std::advance(tiles_it, i);
-        RETURN_CANCEL_OR_ERROR(
-            prepare_tiles(tiles_it->first, &(tiles_it->second)));
+        throw_if_not_ok(prepare_tiles(tiles_it->first, &(tiles_it->second)));
+        throw_if_cancelled();
         return Status::Ok();
       });
 
@@ -621,7 +613,7 @@ Status UnorderedWriter::sort_coords() {
   DomainBuffersView domain_buffs{array_schema_, buffers_};
   if (cell_order != Layout::HILBERT) {  // Row- or col-major
     parallel_sort(
-        storage_manager_->compute_tp(),
+        &resources_.compute_tp(),
         cell_pos_.begin(),
         cell_pos_.end(),
         GlobalCmpQB(domain, domain_buffs));
@@ -629,7 +621,7 @@ Status UnorderedWriter::sort_coords() {
     std::vector<uint64_t> hilbert_values(coords_info_.coords_num_);
     RETURN_NOT_OK(calculate_hilbert_values(domain_buffs, hilbert_values));
     parallel_sort(
-        storage_manager_->compute_tp(),
+        &resources_.compute_tp(),
         cell_pos_.begin(),
         cell_pos_.end(),
         HilbertCmpQB(domain, domain_buffs, hilbert_values));
@@ -732,8 +724,7 @@ Status UnorderedWriter::unordered_write() {
     // The following will make the fragment visible
     URI commit_uri =
         array_->array_directory().get_commit_uri(frag_uri_.value());
-
-    RETURN_NOT_OK(storage_manager_->vfs()->touch(commit_uri));
+    throw_if_not_ok(resources_.vfs().touch(commit_uri));
 
     // Clear some data to prevent it from being serialized.
     cell_pos_.clear();
