@@ -40,6 +40,7 @@
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array/array.h"
 #include "tiledb/sm/array_schema/attribute.h"
+#include "tiledb/sm/array_schema/current_domain.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/dimension_label.h"
 #include "tiledb/sm/array_schema/domain.h"
@@ -67,6 +68,7 @@
 #include "tiledb/sm/filter/xor_filter.h"
 #include "tiledb/sm/misc/constants.h"
 #include "tiledb/sm/serialization/array_schema.h"
+#include "tiledb/sm/serialization/current_domain.h"
 #include "tiledb/sm/serialization/enumeration.h"
 
 #include <cstring>
@@ -76,6 +78,21 @@
 using namespace tiledb::common;
 
 namespace tiledb::sm::serialization {
+class ArraySchemaSerializationException : public StatusException {
+ public:
+  explicit ArraySchemaSerializationException(const std::string& message)
+      : StatusException("[TileDB::Serialization][ArraySchema]", message) {
+  }
+};
+
+class ArraySchemaSerializationDisabledException
+    : public ArraySchemaSerializationException {
+ public:
+  explicit ArraySchemaSerializationDisabledException()
+      : ArraySchemaSerializationException(
+            "Cannot (de)serialize; serialization not enabled.") {
+  }
+};
 
 #ifdef TILEDB_SERIALIZATION
 
@@ -356,7 +373,7 @@ tuple<Status, optional<shared_ptr<FilterPipeline>>> filter_pipeline_from_capnp(
 void attribute_to_capnp(
     const Attribute* attribute, capnp::Attribute::Builder* attribute_builder) {
   if (attribute == nullptr) {
-    throw SerializationStatusException(
+    throw ArraySchemaSerializationException(
         "Error serializing attribute; attribute is null.");
   }
 
@@ -769,7 +786,7 @@ void dimension_label_to_capnp(
   if (dimension_label.uri_is_relative()) {
     dim_label_builder->setUri(dimension_label.uri().to_string());
   } else {
-    throw SerializationStatusException(
+    throw ArraySchemaSerializationException(
         "[Serialization::dimension_label_to_capnp] Serialization of absolute "
         "dimension label URIs not yet implemented.");
   }
@@ -796,7 +813,7 @@ shared_ptr<DimensionLabel> dimension_label_from_capnp(
 
   auto is_relative = dim_label_reader.getRelative();
   if (!is_relative) {
-    throw SerializationStatusException(
+    throw ArraySchemaSerializationException(
         "[Deserialization::dimension_label_from_capnp] Deserialization of "
         "absolute dimension label URIs not yet implemented.");
   }
@@ -912,6 +929,10 @@ Status array_schema_to_capnp(
     }
   }
 
+  auto crd = array_schema.get_current_domain();
+  auto current_domain_builder = array_schema_builder->initCurrentDomain();
+  current_domain_to_capnp(crd, current_domain_builder);
+
   return Status::Ok();
 }
 
@@ -1132,6 +1153,15 @@ shared_ptr<ArraySchema> array_schema_from_capnp(
     name = schema_reader.getName().cStr();
   }
 
+  std::shared_ptr<CurrentDomain> crd;
+  if (schema_reader.hasCurrentDomain()) {
+    crd = current_domain_from_capnp(
+        schema_reader.getCurrentDomain(), domain, memory_tracker);
+  } else {
+    crd = make_shared<CurrentDomain>(
+        memory_tracker, constants::current_domain_version);
+  }
+
   return make_shared<ArraySchema>(
       HERE(),
       uri_deserialized,
@@ -1151,13 +1181,14 @@ shared_ptr<ArraySchema> array_schema_from_capnp(
       cell_var_offsets_filters,
       cell_validity_filters,
       coords_filters,
+      crd,
       memory_tracker);
 }
 
 Status array_schema_serialize(
     const ArraySchema& array_schema,
     SerializationType serialize_type,
-    Buffer* serialized_buffer,
+    SerializationBuffer& serialized_buffer,
     const bool client_side) {
   try {
     ::capnp::MallocMessageBuilder message;
@@ -1166,27 +1197,16 @@ Status array_schema_serialize(
     RETURN_NOT_OK(
         array_schema_to_capnp(array_schema, &arraySchemaBuilder, client_side));
 
-    serialized_buffer->reset_size();
-    serialized_buffer->reset_offset();
-
     switch (serialize_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(arraySchemaBuilder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        RETURN_NOT_OK(serialized_buffer->realloc(json_len + 1));
-        RETURN_NOT_OK(serialized_buffer->write(capnp_json.cStr(), json_len));
-        RETURN_NOT_OK(serialized_buffer->write(&nul, 1));
+        serialized_buffer.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        RETURN_NOT_OK(serialized_buffer->realloc(nbytes));
-        RETURN_NOT_OK(serialized_buffer->write(message_chars.begin(), nbytes));
+        serialized_buffer.assign(protomessage.asChars());
         break;
       }
       default: {
@@ -1210,7 +1230,7 @@ Status array_schema_serialize(
 
 shared_ptr<ArraySchema> array_schema_deserialize(
     SerializationType serialize_type,
-    const Buffer& serialized_buffer,
+    span<const char> serialized_buffer,
     shared_ptr<MemoryTracker> memory_tracker) {
   capnp::ArraySchema::Reader array_schema_reader;
   ::capnp::MallocMessageBuilder message_builder;
@@ -1260,7 +1280,7 @@ Status nonempty_domain_serialize(
     const void* nonempty_domain,
     bool is_empty,
     SerializationType serialize_type,
-    Buffer* serialized_buffer) {
+    SerializationBuffer& serialized_buffer) {
   if (!is_empty && nonempty_domain == nullptr)
     return LOG_STATUS(Status_SerializationError(
         "Error serializing nonempty domain; nonempty domain is null."));
@@ -1277,28 +1297,16 @@ Status nonempty_domain_serialize(
           subarray_builder, dimension, nonempty_domain));
     }
 
-    // Copy to buffer
-    serialized_buffer->reset_size();
-    serialized_buffer->reset_offset();
-
     switch (serialize_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        RETURN_NOT_OK(serialized_buffer->realloc(json_len + 1));
-        RETURN_NOT_OK(serialized_buffer->write(capnp_json.cStr(), json_len));
-        RETURN_NOT_OK(serialized_buffer->write(&nul, 1));
+        serialized_buffer.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        RETURN_NOT_OK(serialized_buffer->realloc(nbytes));
-        RETURN_NOT_OK(serialized_buffer->write(message_chars.begin(), nbytes));
+        serialized_buffer.assign(protomessage.asChars());
         break;
       }
       default: {
@@ -1323,7 +1331,7 @@ Status nonempty_domain_serialize(
 
 Status nonempty_domain_deserialize(
     const Dimension* dimension,
-    const Buffer& serialized_buffer,
+    span<const char> serialized_buffer,
     SerializationType serialize_type,
     void* nonempty_domain,
     bool* is_empty) {
@@ -1398,7 +1406,7 @@ Status nonempty_domain_serialize(
     const void* nonempty_domain,
     bool is_empty,
     SerializationType serialize_type,
-    Buffer* serialized_buffer) {
+    SerializationBuffer& serialized_buffer) {
   if (!is_empty && nonempty_domain == nullptr)
     return LOG_STATUS(Status_SerializationError(
         "Error serializing nonempty domain; nonempty domain is null."));
@@ -1417,28 +1425,16 @@ Status nonempty_domain_serialize(
           utils::serialize_subarray(subarray_builder, schema, nonempty_domain));
     }
 
-    // Copy to buffer
-    serialized_buffer->reset_size();
-    serialized_buffer->reset_offset();
-
     switch (serialize_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        RETURN_NOT_OK(serialized_buffer->realloc(json_len + 1));
-        RETURN_NOT_OK(serialized_buffer->write(capnp_json.cStr(), json_len));
-        RETURN_NOT_OK(serialized_buffer->write(&nul, 1));
+        serialized_buffer.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        RETURN_NOT_OK(serialized_buffer->realloc(nbytes));
-        RETURN_NOT_OK(serialized_buffer->write(message_chars.begin(), nbytes));
+        serialized_buffer.assign(protomessage.asChars());
         break;
       }
       default: {
@@ -1463,7 +1459,7 @@ Status nonempty_domain_serialize(
 
 Status nonempty_domain_deserialize(
     const Array* array,
-    const Buffer& serialized_buffer,
+    span<const char> serialized_buffer,
     SerializationType serialize_type,
     void* nonempty_domain,
     bool* is_empty) {
@@ -1542,7 +1538,9 @@ Status nonempty_domain_deserialize(
 }
 
 Status nonempty_domain_serialize(
-    Array* array, SerializationType serialize_type, Buffer* serialized_buffer) {
+    Array* array,
+    SerializationType serialize_type,
+    SerializationBuffer& serialized_buffer) {
   try {
     // Serialize
     ::capnp::MallocMessageBuilder message;
@@ -1550,28 +1548,16 @@ Status nonempty_domain_serialize(
 
     RETURN_NOT_OK(utils::serialize_non_empty_domain(builder, array));
 
-    // Copy to buffer
-    serialized_buffer->reset_size();
-    serialized_buffer->reset_offset();
-
     switch (serialize_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        RETURN_NOT_OK(serialized_buffer->realloc(json_len + 1));
-        RETURN_NOT_OK(serialized_buffer->write(capnp_json.cStr(), json_len));
-        RETURN_NOT_OK(serialized_buffer->write(&nul, 1));
+        serialized_buffer.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        RETURN_NOT_OK(serialized_buffer->realloc(nbytes));
-        RETURN_NOT_OK(serialized_buffer->write(message_chars.begin(), nbytes));
+        serialized_buffer.assign(protomessage.asChars());
         break;
       }
       default: {
@@ -1596,7 +1582,7 @@ Status nonempty_domain_serialize(
 
 Status nonempty_domain_deserialize(
     Array* array,
-    const Buffer& serialized_buffer,
+    span<const char> serialized_buffer,
     SerializationType serialize_type) {
   try {
     switch (serialize_type) {
@@ -1644,230 +1630,52 @@ Status nonempty_domain_deserialize(
   return Status::Ok();
 }
 
-Status max_buffer_sizes_serialize(
-    Array* array,
-    const void* subarray,
-    SerializationType serialize_type,
-    Buffer* serialized_buffer) {
-  const auto& schema = array->array_schema_latest();
-
-  try {
-    // Serialize
-    ::capnp::MallocMessageBuilder message;
-    auto builder = message.initRoot<capnp::MaxBufferSizes>();
-
-    // Get all attribute names including coords
-    const auto& attrs = schema.attributes();
-    std::set<std::string> attr_names;
-    attr_names.insert(constants::coords);
-    for (const auto& a : attrs)
-      attr_names.insert(a->name());
-
-    // Get max buffer size for each attribute from the given Array instance
-    // and serialize it.
-    auto max_buffer_sizes_builder =
-        builder.initMaxBufferSizes(attr_names.size());
-    size_t i = 0;
-    for (const auto& attr_name : attr_names) {
-      bool var_size =
-          attr_name != constants::coords && schema.var_size(attr_name);
-      auto max_buffer_size_builder = max_buffer_sizes_builder[i++];
-      max_buffer_size_builder.setAttribute(attr_name);
-
-      if (var_size) {
-        uint64_t offset_bytes, data_bytes;
-        RETURN_NOT_OK(array->get_max_buffer_size(
-            attr_name.c_str(), subarray, &offset_bytes, &data_bytes));
-        max_buffer_size_builder.setOffsetBytes(offset_bytes);
-        max_buffer_size_builder.setDataBytes(data_bytes);
-      } else {
-        uint64_t data_bytes;
-        RETURN_NOT_OK(array->get_max_buffer_size(
-            attr_name.c_str(), subarray, &data_bytes));
-        max_buffer_size_builder.setOffsetBytes(0);
-        max_buffer_size_builder.setDataBytes(data_bytes);
-      }
-    }
-
-    // Copy to buffer
-    serialized_buffer->reset_size();
-    serialized_buffer->reset_offset();
-
-    switch (serialize_type) {
-      case SerializationType::JSON: {
-        ::capnp::JsonCodec json;
-        kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        RETURN_NOT_OK(serialized_buffer->realloc(json_len + 1));
-        RETURN_NOT_OK(serialized_buffer->write(capnp_json.cStr(), json_len));
-        RETURN_NOT_OK(serialized_buffer->write(&nul, 1));
-        break;
-      }
-      case SerializationType::CAPNP: {
-        kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        RETURN_NOT_OK(serialized_buffer->realloc(nbytes));
-        RETURN_NOT_OK(serialized_buffer->write(message_chars.begin(), nbytes));
-        break;
-      }
-      default: {
-        return LOG_STATUS(Status_SerializationError(
-            "Error serializing max buffer sizes; Unknown serialization type "
-            "passed"));
-      }
-    }
-
-  } catch (kj::Exception& e) {
-    return LOG_STATUS(Status_SerializationError(
-        "Error serializing max buffer sizes; kj::Exception: " +
-        std::string(e.getDescription().cStr())));
-  } catch (std::exception& e) {
-    return LOG_STATUS(Status_SerializationError(
-        "Error serializing max buffer sizes; exception " +
-        std::string(e.what())));
-  }
-
-  return Status::Ok();
-}
-
-Status max_buffer_sizes_deserialize(
-    const ArraySchema& schema,
-    const Buffer& serialized_buffer,
-    SerializationType serialize_type,
-    std::unordered_map<std::string, std::pair<uint64_t, uint64_t>>*
-        buffer_sizes) {
-  try {
-    switch (serialize_type) {
-      case SerializationType::JSON: {
-        ::capnp::JsonCodec json;
-        ::capnp::MallocMessageBuilder message_builder;
-        auto builder = message_builder.initRoot<capnp::MaxBufferSizes>();
-        json.decode(
-            kj::StringPtr(static_cast<const char*>(serialized_buffer.data())),
-            builder);
-        auto reader = builder.asReader();
-
-        // Deserialize
-        auto max_buffer_sizes_reader = reader.getMaxBufferSizes();
-        const size_t num_max_buffer_sizes = max_buffer_sizes_reader.size();
-        for (size_t i = 0; i < num_max_buffer_sizes; i++) {
-          auto max_buffer_size_reader = max_buffer_sizes_reader[i];
-          std::string attribute = max_buffer_size_reader.getAttribute();
-          uint64_t offset_size = max_buffer_size_reader.getOffsetBytes();
-          uint64_t data_size = max_buffer_size_reader.getDataBytes();
-
-          if (attribute == constants::coords || !schema.var_size(attribute)) {
-            (*buffer_sizes)[attribute] = std::make_pair(data_size, 0);
-          } else {
-            (*buffer_sizes)[attribute] = std::make_pair(offset_size, data_size);
-          }
-        }
-
-        break;
-      }
-      case SerializationType::CAPNP: {
-        const auto mBytes =
-            reinterpret_cast<const kj::byte*>(serialized_buffer.data());
-        ::capnp::FlatArrayMessageReader msg_reader(kj::arrayPtr(
-            reinterpret_cast<const ::capnp::word*>(mBytes),
-            serialized_buffer.size() / sizeof(::capnp::word)));
-        auto reader = msg_reader.getRoot<capnp::MaxBufferSizes>();
-
-        // Deserialize
-        auto max_buffer_sizes_reader = reader.getMaxBufferSizes();
-        const size_t num_max_buffer_sizes = max_buffer_sizes_reader.size();
-        for (size_t i = 0; i < num_max_buffer_sizes; i++) {
-          auto max_buffer_size_reader = max_buffer_sizes_reader[i];
-          std::string attribute = max_buffer_size_reader.getAttribute();
-          uint64_t offset_size = max_buffer_size_reader.getOffsetBytes();
-          uint64_t data_size = max_buffer_size_reader.getDataBytes();
-
-          if (attribute == constants::coords || !schema.var_size(attribute)) {
-            (*buffer_sizes)[attribute] = std::make_pair(data_size, 0);
-          } else {
-            (*buffer_sizes)[attribute] = std::make_pair(offset_size, data_size);
-          }
-        }
-
-        break;
-      }
-      default: {
-        return LOG_STATUS(Status_SerializationError(
-            "Error deserializing max buffer sizes; Unknown serialization type "
-            "passed"));
-      }
-    }
-  } catch (kj::Exception& e) {
-    return LOG_STATUS(Status_SerializationError(
-        "Error deserializing max buffer sizes; kj::Exception: " +
-        std::string(e.getDescription().cStr())));
-  } catch (std::exception& e) {
-    return LOG_STATUS(Status_SerializationError(
-        "Error deserializing max buffer sizes; exception " +
-        std::string(e.what())));
-  }
-
-  return Status::Ok();
-}
-
 void load_array_schema_request_to_capnp(
     capnp::LoadArraySchemaRequest::Builder& builder,
     const Config& config,
     const LoadArraySchemaRequest& req) {
   auto config_builder = builder.initConfig();
   throw_if_not_ok(config_to_capnp(config, &config_builder));
-  builder.setIncludeEnumerations(req.include_enumerations());
+  // This boolean is only serialized to support clients using TileDB < 2.26.
+  // Future options should only be serialized within the Config object above.
+  builder.setIncludeEnumerations(req.include_enumerations_latest_schema());
 }
 
 void serialize_load_array_schema_request(
     const Config& config,
     const LoadArraySchemaRequest& req,
     SerializationType serialization_type,
-    Buffer& data) {
+    SerializationBuffer& data) {
   try {
     ::capnp::MallocMessageBuilder message;
     auto builder = message.initRoot<capnp::LoadArraySchemaRequest>();
     load_array_schema_request_to_capnp(builder, config, req);
 
-    data.reset_size();
-    data.reset_offset();
-
     switch (serialization_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        throw_if_not_ok(data.realloc(json_len + 1));
-        throw_if_not_ok(data.write(capnp_json.cStr(), json_len));
-        throw_if_not_ok(data.write(&nul, 1));
+        data.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        throw_if_not_ok(data.realloc(nbytes));
-        throw_if_not_ok(data.write(message_chars.begin(), nbytes));
+        data.assign(protomessage.asChars());
         break;
       }
       default: {
-        throw Status_SerializationError(
+        throw ArraySchemaSerializationException(
             "Error serializing load array schema request; "
             "Unknown serialization type passed");
       }
     }
 
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error serializing load array schema request; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error serializing load array schema request; exception " +
         std::string(e.what()));
   }
@@ -1875,11 +1683,19 @@ void serialize_load_array_schema_request(
 
 LoadArraySchemaRequest load_array_schema_request_from_capnp(
     capnp::LoadArraySchemaRequest::Reader& reader) {
-  return LoadArraySchemaRequest(reader.getIncludeEnumerations());
+  tdb_unique_ptr<Config> decoded_config = nullptr;
+  if (reader.hasConfig()) {
+    throw_if_not_ok(config_from_capnp(reader.getConfig(), &decoded_config));
+  } else {
+    decoded_config.reset(tdb_new(Config));
+  }
+  // We intentionally do not use the includeEnumerations field, as it is stored
+  // in the Config and set using the LoadArraySchemaRequest constructor.
+  return LoadArraySchemaRequest(*decoded_config);
 }
 
 LoadArraySchemaRequest deserialize_load_array_schema_request(
-    SerializationType serialization_type, const Buffer& data) {
+    SerializationType serialization_type, span<const char> data) {
   try {
     switch (serialization_type) {
       case SerializationType::JSON: {
@@ -1887,8 +1703,7 @@ LoadArraySchemaRequest deserialize_load_array_schema_request(
         ::capnp::MallocMessageBuilder message_builder;
         auto builder =
             message_builder.initRoot<capnp::LoadArraySchemaRequest>();
-        json.decode(
-            kj::StringPtr(static_cast<const char*>(data.data())), builder);
+        json.decode(kj::StringPtr(data.data()), builder);
         auto reader = builder.asReader();
         return load_array_schema_request_from_capnp(reader);
       }
@@ -1901,89 +1716,117 @@ LoadArraySchemaRequest deserialize_load_array_schema_request(
         return load_array_schema_request_from_capnp(reader);
       }
       default: {
-        throw Status_SerializationError(
+        throw ArraySchemaSerializationException(
             "Error deserializing load array schema request; "
             "Unknown serialization type passed");
       }
     }
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error deserializing load array schema request; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error deserializing load array schema request; exception " +
         std::string(e.what()));
   }
 }
 
 void load_array_schema_response_to_capnp(
-    capnp::LoadArraySchemaResponse::Builder& builder,
-    const ArraySchema& schema) {
+    capnp::LoadArraySchemaResponse::Builder& builder, const Array& array) {
   auto schema_builder = builder.initSchema();
-  throw_if_not_ok(array_schema_to_capnp(schema, &schema_builder, false));
+  throw_if_not_ok(array_schema_to_capnp(
+      array.array_schema_latest(), &schema_builder, false));
+
+  const auto& array_schemas_all = array.array_schemas_all();
+  auto array_schemas_all_builder = builder.initArraySchemasAll();
+  auto entries_builder =
+      array_schemas_all_builder.initEntries(array_schemas_all.size());
+  uint64_t i = 0;
+  for (const auto& schema : array_schemas_all) {
+    auto entry = entries_builder[i++];
+    entry.setKey(schema.first);
+    auto schema_entry_builder = entry.initValue();
+    throw_if_not_ok(
+        array_schema_to_capnp(*(schema.second), &schema_entry_builder, false));
+  }
 }
 
 void serialize_load_array_schema_response(
-    const ArraySchema& schema,
+    const Array& array,
     SerializationType serialization_type,
-    Buffer& data) {
+    SerializationBuffer& data) {
   try {
     ::capnp::MallocMessageBuilder message;
     auto builder = message.initRoot<capnp::LoadArraySchemaResponse>();
-    load_array_schema_response_to_capnp(builder, schema);
-
-    data.reset_size();
-    data.reset_offset();
+    load_array_schema_response_to_capnp(builder, array);
 
     switch (serialization_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        throw_if_not_ok(data.realloc(json_len + 1));
-        throw_if_not_ok(data.write(capnp_json.cStr(), json_len));
-        throw_if_not_ok(data.write(&nul, 1));
+        data.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        throw_if_not_ok(data.realloc(nbytes));
-        throw_if_not_ok(data.write(message_chars.begin(), nbytes));
+        data.assign(protomessage.asChars());
         break;
       }
       default: {
-        throw Status_SerializationError(
+        throw ArraySchemaSerializationException(
             "Error serializing load array schema response; "
             "Unknown serialization type passed");
       }
     }
 
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error serializing load array schema response; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error serializing load array schema response; exception " +
         std::string(e.what()));
   }
 }
 
-shared_ptr<ArraySchema> load_array_schema_response_from_capnp(
+std::tuple<
+    shared_ptr<ArraySchema>,
+    std::unordered_map<std::string, shared_ptr<ArraySchema>>>
+load_array_schema_response_from_capnp(
+    const URI& uri,
     capnp::LoadArraySchemaResponse::Reader& reader,
     shared_ptr<MemoryTracker> memory_tracker) {
   auto schema_reader = reader.getSchema();
-  return array_schema_from_capnp(schema_reader, URI(), memory_tracker);
+  auto schema = array_schema_from_capnp(schema_reader, URI(), memory_tracker);
+  schema->set_array_uri(uri);
+
+  std::unordered_map<std::string, shared_ptr<ArraySchema>> all_schemas;
+  if (reader.hasArraySchemasAll()) {
+    auto all_schemas_reader = reader.getArraySchemasAll();
+
+    if (all_schemas_reader.hasEntries()) {
+      auto entries = all_schemas_reader.getEntries();
+      for (auto array_schema_build : entries) {
+        auto schema_entry = array_schema_from_capnp(
+            array_schema_build.getValue(), schema->array_uri(), memory_tracker);
+        schema_entry->set_array_uri(schema->array_uri());
+        all_schemas[array_schema_build.getKey()] = schema_entry;
+      }
+    }
+  }
+  return {schema, all_schemas};
 }
 
-shared_ptr<ArraySchema> deserialize_load_array_schema_response(
+std::tuple<
+    shared_ptr<ArraySchema>,
+    std::unordered_map<std::string, shared_ptr<ArraySchema>>>
+deserialize_load_array_schema_response(
+    const URI& uri,
+    const Config& config,
     SerializationType serialization_type,
-    const Buffer& data,
+    span<const char> data,
     shared_ptr<MemoryTracker> memory_tracker) {
   try {
     switch (serialization_type) {
@@ -1992,31 +1835,41 @@ shared_ptr<ArraySchema> deserialize_load_array_schema_response(
         ::capnp::MallocMessageBuilder message_builder;
         auto builder =
             message_builder.initRoot<capnp::LoadArraySchemaResponse>();
-        json.decode(
-            kj::StringPtr(static_cast<const char*>(data.data())), builder);
+        json.decode(kj::StringPtr(data.data()), builder);
         auto reader = builder.asReader();
-        return load_array_schema_response_from_capnp(reader, memory_tracker);
+        return load_array_schema_response_from_capnp(
+            uri, reader, memory_tracker);
       }
       case SerializationType::CAPNP: {
+        // Set traversal limit from config
+        uint64_t limit =
+            config.get<uint64_t>("rest.capnp_traversal_limit").value();
+        ::capnp::ReaderOptions readerOptions;
+        // capnp uses the limit in words
+        readerOptions.traversalLimitInWords = limit / sizeof(::capnp::word);
+
         const auto mBytes = reinterpret_cast<const kj::byte*>(data.data());
-        ::capnp::FlatArrayMessageReader array_reader(kj::arrayPtr(
-            reinterpret_cast<const ::capnp::word*>(mBytes),
-            data.size() / sizeof(::capnp::word)));
+        ::capnp::FlatArrayMessageReader array_reader(
+            kj::arrayPtr(
+                reinterpret_cast<const ::capnp::word*>(mBytes),
+                data.size() / sizeof(::capnp::word)),
+            readerOptions);
         auto reader = array_reader.getRoot<capnp::LoadArraySchemaResponse>();
-        return load_array_schema_response_from_capnp(reader, memory_tracker);
+        return load_array_schema_response_from_capnp(
+            uri, reader, memory_tracker);
       }
       default: {
-        throw Status_SerializationError(
+        throw ArraySchemaSerializationException(
             "Error deserializing load array schema response; "
             "Unknown serialization type passed");
       }
     }
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error deserializing load array schema response; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw ArraySchemaSerializationException(
         "Error deserializing load array schema response; exception " +
         std::string(e.what()));
   }
@@ -2025,76 +1878,69 @@ shared_ptr<ArraySchema> deserialize_load_array_schema_response(
 #else
 
 Status array_schema_serialize(
-    const ArraySchema&, SerializationType, Buffer*, const bool) {
+    const ArraySchema&, SerializationType, SerializationBuffer&, const bool) {
   return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
 shared_ptr<ArraySchema> array_schema_deserialize(
-    SerializationType, const Buffer&, shared_ptr<MemoryTracker>) {
+    SerializationType, span<const char>, shared_ptr<MemoryTracker>) {
   throw StatusException(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
-Status nonempty_domain_serialize(Array*, SerializationType, Buffer*) {
-  return LOG_STATUS(Status_SerializationError(
-      "Cannot serialize; serialization not enabled."));
-}
-
-Status nonempty_domain_deserialize(Array*, const Buffer&, SerializationType) {
-  return LOG_STATUS(Status_SerializationError(
-      "Cannot serialize; serialization not enabled."));
-}
-
 Status nonempty_domain_serialize(
-    const Array*, const void*, bool, SerializationType, Buffer*) {
+    Array*, SerializationType, SerializationBuffer&) {
   return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
 Status nonempty_domain_deserialize(
-    const Array*, const Buffer&, SerializationType, void*, bool*) {
+    Array*, span<const char>, SerializationType) {
   return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
-Status max_buffer_sizes_serialize(
-    Array*, const void*, SerializationType, Buffer*) {
+Status nonempty_domain_serialize(
+    const Array*, const void*, bool, SerializationType, SerializationBuffer&) {
   return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
-Status max_buffer_sizes_deserialize(
-    const ArraySchema&,
-    const Buffer&,
-    SerializationType,
-    std::unordered_map<std::string, std::pair<uint64_t, uint64_t>>*) {
+Status nonempty_domain_deserialize(
+    const Array*, span<const char>, SerializationType, void*, bool*) {
   return LOG_STATUS(Status_SerializationError(
       "Cannot serialize; serialization not enabled."));
 }
 
 void serialize_load_array_schema_request(
-    const Config&, const LoadArraySchemaRequest&, SerializationType, Buffer&) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    const Config&,
+    const LoadArraySchemaRequest&,
+    SerializationType,
+    SerializationBuffer&) {
+  throw ArraySchemaSerializationDisabledException();
 }
 
 LoadArraySchemaRequest deserialize_load_array_schema_request(
-    SerializationType, const Buffer&) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    SerializationType, span<const char>) {
+  throw ArraySchemaSerializationDisabledException();
 }
 
 void serialize_load_array_schema_response(
-    const ArraySchema&, SerializationType, Buffer&) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    const Array&, SerializationType, SerializationBuffer&) {
+  throw ArraySchemaSerializationDisabledException();
 }
 
-shared_ptr<ArraySchema> deserialize_load_array_schema_response(
-    SerializationType, const Buffer&, shared_ptr<MemoryTracker>) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+std::tuple<
+    shared_ptr<ArraySchema>,
+    std::unordered_map<std::string, shared_ptr<ArraySchema>>>
+deserialize_load_array_schema_response(
+    const URI&,
+    const Config&,
+    SerializationType,
+    span<const char>,
+    shared_ptr<MemoryTracker>) {
+  throw ArraySchemaSerializationDisabledException();
 }
 
 #endif  // TILEDB_SERIALIZATION
